@@ -15,12 +15,8 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from itools
-from utils import get_version, get_abspath
-from gettext import register_domain
+from core import get_version
 
 
 __version__ = get_version()
 
-# Register the itools domain
-path = get_abspath('locale')
-register_domain('itools', path)
