@@ -16,12 +16,12 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from itools
+from itools.core import add_type
 from csv_ import CSVFile, Row
-from itools.utils import add_type
-from memory import MemoryCatalog
 from parser import parse
-from table import Table, Record, Property, UniqueError
+from table import Table, Record, UniqueError
 from table import parse_table, fold_line, escape_data, is_multilingual
+from table import Property, property_to_str, deserialize_parameters
 
 
 __all__ = [
@@ -30,16 +30,17 @@ __all__ = [
     # Classes
     'CSVFile',
     'Row',
-    'MemoryCatalog',
     # The Table handler (a kind of CSV on steroids)
     'Table',
     'Record',
     'Property',
+    'property_to_str',
     'UniqueError',
     'parse_table',
     'fold_line',
     'escape_data',
     'is_multilingual',
+    'deserialize_parameters',
     ]
 
 
