@@ -21,12 +21,12 @@
 from datetime import datetime
 
 # Import from itools
-from csv_ import CSVFile
+from itools.core import merge_dicts
 from itools.datatypes import DateTime, String, Integer, Unicode
 from itools.handlers import File
-from itools import vfs
-from itools.xapian import AndQuery, PhraseQuery, get_field
-from memory import MemoryCatalog
+from itools.xapian import make_catalog
+from itools.xapian import PhraseQuery, CatalogAware
+from csv_ import CSVFile
 from parser import parse
 
 
@@ -180,10 +180,16 @@ def get_tokens(property):
         # param-name ended, param-value beginning
         elif status == 3:
             if c == '"':
-                param_value = c
+                param_value = ''
                 status = 4
-            elif c in (';', ':', ',') :
-                raise SyntaxError, error1 % (c, status)
+            elif c == ':':
+                parameters[param_name].append('')
+                status = 7
+            elif c == ';':
+                parameters[param_name].append('')
+                status = 1
+            elif c == ',':
+                parameters[param_name].append('')
             else:
                 param_value = c
                 status = 5
@@ -194,7 +200,6 @@ def get_tokens(property):
                 if last == '"':
                     raise SyntaxError, error2 % (c, status)
                 last = '"'
-                param_value += c
                 status = 6
             else:
                 param_value += c
@@ -269,6 +274,23 @@ def is_multilingual(datatype):
 
 
 
+def deserialize_parameters(parameters, schema, default=String(multiple=True)):
+    for name in parameters:
+        value = parameters[name]
+        datatype = schema.get(name, default)
+        # Decode
+        value = [ datatype.decode(x) for x in value ]
+        # Multiple or single
+        if not datatype.multiple:
+            if len(value) > 1:
+                msg = 'parameter "%s" must be a singleton'
+                raise ValueError, msg % name
+            value = value[0]
+        # Update
+        parameters[name] = value
+
+
+
 ###########################################################################
 # UniqueError
 ###########################################################################
@@ -282,8 +304,9 @@ class UniqueError(ValueError):
 
 
     def __str__(self):
-        return (u'Error: Field "$field" must be unique, '
-                u'value "$value" is already used.')
+        return (
+            u'the "{field}" field must be unique, the "{value}" value is '
+            u' already used.')
 
 
 ###########################################################################
@@ -301,17 +324,88 @@ class Property(object):
 
     def __init__(self, value, **kw):
         self.value = value
-        self.parameters = kw
+        self.parameters = kw or None
+
+
+    def get_parameter(self, name, default=None):
+        if self.parameters is None:
+            return default
+        return self.parameters.get(name, default)
+
+
+    def set_parameter(self, name, value):
+        if self.parameters is None:
+            self.parameters = {}
+        self.parameters[name] = value
 
 
 
-class Record(list):
-
-    __slots__ = ['id']
+def encode_param_value(p_name, p_value, p_datatype):
+    p_value = p_datatype.encode(p_value)
+    if '"' in p_value:
+        error = 'the "%s" parameter contains a double quote'
+        raise ValueError, error % p_name
+    if ';' in p_value or ':' in p_value or ',' in p_value:
+        return '"%s"' % p_value
+    return p_value
 
 
-    def __init__(self, id):
+
+def property_to_str(name, property, datatype, p_schema, encoding='utf-8'):
+    """This method serializes the given property to a byte string:
+
+      name[;parameters]=value
+
+    The given datatype is used to serialize the property value.  The given
+    'p_schema' describes the parameters.
+    """
+    # Parameters
+    if property.parameters:
+        p_names = property.parameters.keys()
+        p_names.sort()
+    else:
+        p_names = []
+
+    parameters = []
+    for p_name in p_names:
+        p_value = property.parameters[p_name]
+        # Find out the datatype for the parameter
+        p_datatype = p_schema.get(p_name)
+        if not p_datatype:
+            p_datatype = String(multiple=True)
+        # Serialize the parameter
+        # FIXME Use the encoding
+        if getattr(p_datatype, 'multiple', False):
+            p_value = [
+                encode_param_value(p_name, x, p_datatype) for x in p_value ]
+            p_value = ','.join(p_value)
+        else:
+            p_value = encode_param_value(p_name, p_value, p_datatype)
+        parameters.append(';%s=%s' % (p_name, p_value))
+    parameters = ''.join(parameters)
+
+    # Value
+    if isinstance(datatype, Unicode):
+        value = datatype.encode(property.value, encoding=encoding)
+    else:
+        value = datatype.encode(property.value)
+    value = escape_data(value)
+
+    # Ok
+    property = '%s%s:%s\n' % (name, parameters, value)
+    return fold_line(property)
+
+
+
+class Record(list, CatalogAware):
+
+    __slots__ = ['id', 'record_properties']
+
+
+    def __init__(self, id, record_properties):
         self.id = id
+        self.record_properties  = record_properties
+        self.append({})
 
 
     def __getattr__(self, name):
@@ -348,6 +442,13 @@ class Record(list):
         return property.value
 
 
+    def get_catalog_values(self):
+        values = {'__id__': self.id}
+        for name in self.record_properties.iterkeys():
+            values[name] = self.get_value(name)
+        return values
+
+
 
 class Table(File):
 
@@ -358,14 +459,14 @@ class Table(File):
     # Hash with field names and its types
     # Example: {'firstname': Unicode, 'lastname': Unicode, 'age': Integer}
     # To index some fields the schema should be declared as:
-    # record_schema = {'firstname': Unicode, 'lastname': Unicode,
-    #                  'age': Integer(index='<analyser>')}
-    # where <analyser> is an itools.xapian analyser or derivate: keyword,
-    # book, text, path.
+    # record_properties = {
+    #     'firstname': Unicode,
+    #     'lastname': Unicode,
+    #     'age': Integer(is_indexed=True)}
     #######################################################################
     schema = {}
-    record_schema = {}
-    parameters_schema = {
+    record_properties = {}
+    record_parameters = {
         'language': String(multiple=False)}
 
 
@@ -382,27 +483,16 @@ class Table(File):
         # Record schema
         if name == 'ts':
             return DateTime(multiple=False)
-        if name in self.record_schema:
-            return self.record_schema[name]
+        if name in self.record_properties:
+            return self.record_properties[name]
         # FIXME Probably we should raise an exception here
         return String(multiple=True)
 
 
-    def get_parameter_datatype(self, name):
-        # Record schema
-        schema = self.parameters_schema
-        if name in schema:
-            return schema[name]
-        return String
-
-
-    def properties_to_dict(self, properties, version=None, first=False):
+    def properties_to_dict(self, properties, version, first=False):
         """Add the given "properties" as Property objects or Property objects
         list to the given dictionnary "version".
         """
-        if version is None:
-            version = {}
-
         # The variable 'first' defines whether we are talking about the
         # table properties (True) or a about records (False).
         if first is True:
@@ -431,31 +521,31 @@ class Table(File):
                     version[name] = [to_property(value)]
             else:
                 version[name] = to_property(value)
-        return version
 
 
     #######################################################################
     # Handlers
     #######################################################################
+    clone_exclude = File.clone_exclude | frozenset(['catalog'])
+
+
     def reset(self):
         self.properties = None
         self.records = []
-        self.added_properties = []
+        self.changed_properties = False
         self.added_records = []
         self.removed_records = []
         # The catalog (for index and search)
-        self.catalog = MemoryCatalog()
-        for name, datatype in self.record_schema.items():
-            index = getattr(datatype, 'index', None)
-            if index is not None:
-                field = get_field(index)
-                self.catalog.add_index(name, field)
+        fields = merge_dicts(
+            self.record_properties,
+            __id__=Integer(is_key_field=True, is_stored=True, is_indexed=True))
+        self.catalog = make_catalog(None, fields)
 
 
     def new(self):
         # Add the properties record
-        properties = self.record_class(-1)
-        properties.append({'ts': Property(datetime.now())})
+        properties = self.record_class(-1, self.record_properties)
+        properties[0] = {'ts': Property(datetime.now())}
         self.properties = properties
 
 
@@ -463,6 +553,8 @@ class Table(File):
         # Load the records
         records = self.records
         properties = self.properties
+        record_properties = self.record_properties
+
         n = 0
         version = None
         for name, value, parameters in parse_table(file.read()):
@@ -475,13 +567,13 @@ class Table(File):
                 if uid == -1:
                     # Tale properties
                     if properties is None:
-                        properties = self.record_class(uid)
+                        properties = self.record_class(uid, record_properties)
                         self.properties = properties
                     record = properties
                 elif uid >= n:
                     # New record
                     records.extend([None] * (uid - n))
-                    record = self.record_class(uid)
+                    record = self.record_class(uid, record_properties)
                     records.append(record)
                     n = uid + 1
                 else:
@@ -496,11 +588,7 @@ class Table(File):
                         records[uid] = None
                         record = None
                 else:
-                    seq = int(seq)
-                    if seq > len(record):
-                        msg = 'unexpected sequence "%s" for record "%s"'
-                        raise ValueError, msg % (seq, uid)
-                    record.append(version)
+                    record[0] = version
                 # Table or record schema
                 if uid == -1:
                     get_datatype = self.get_datatype
@@ -509,19 +597,7 @@ class Table(File):
                 continue
 
             # Deserialize the parameters
-            for param_name in parameters.keys():
-                param_value = parameters[param_name]
-                param_type = self.get_parameter_datatype(param_name)
-                # Decode
-                param_value = [ param_type.decode(x) for x in param_value ]
-                # Multiple or single
-                if not param_type.multiple:
-                    if len(param_value) > 1:
-                        msg = 'parameter "%s" must be a singleton'
-                        raise ValueError, msg % param_name
-                    param_value = param_value[0]
-                # Update
-                parameters[param_name] = param_value
+            deserialize_parameters(parameters, self.record_parameters)
 
             # Timestamp (ts), Schema, or Something else
             datatype = get_datatype(name)
@@ -536,11 +612,11 @@ class Table(File):
         # Index the records
         for record in records:
             if record is not None:
-                self.catalog.index_document(record, record.id)
+                self.catalog.index_document(record)
 
 
-    def _version_to_str(self, id, seq, version):
-        lines = ['id:%d/%d\n' % (id, seq)]
+    def _version_to_str(self, id, version):
+        lines = ['id:%d/0\n' % id]
         names = version.keys()
         names.sort()
         # Table or record schema
@@ -550,6 +626,7 @@ class Table(File):
             get_datatype = self.get_record_datatype
 
         # Loop
+        p_schema = self.record_parameters
         for name in names:
             datatype = get_datatype(name)
             if getattr(datatype, 'multiple', False) is True:
@@ -559,26 +636,8 @@ class Table(File):
             for property in properties:
                 if property.value is None:
                     continue
-                lines.append(name)
-                # Parameters
-                pnames = property.parameters.keys()
-                pnames.sort()
-                for pname in pnames:
-                    pvalues = property.parameters[pname]
-                    pdatatype = self.get_parameter_datatype(pname)
-                    if pdatatype.multiple:
-                        pvalues = [ pdatatype.encode(x) for x in pvalues ]
-                        pvalues = ','.join(pvalues)
-                    else:
-                        pvalues = pdatatype.encode(pvalues)
-                    lines.append(';%s=%s' % (pname, pvalues))
-                # Value
-                value = datatype.encode(property.value)
-                if isinstance(value, Integer):
-                    value = str(value)
-                # Escape the value
-                value = escape_data(value)
-                lines.append(':%s\n' % fold_line(value))
+                property = property_to_str(name, property, datatype, p_schema)
+                lines.append(property)
 
         # Ok
         lines.append('\n')
@@ -590,19 +649,15 @@ class Table(File):
         id = 0
         # Properties record
         if self.properties is not None:
-            seq = 0
-            for version in self.properties:
-                version = self._version_to_str(-1, seq, version)
-                lines.append(version)
-                seq += 1
+            version = self.properties[0]
+            version = self._version_to_str(-1, version)
+            lines.append(version)
         # Common record
         for record in self.records:
             if record is not None:
-                seq = 0
-                for version in record:
-                    version = self._version_to_str(id, seq, version)
-                    lines.append(version)
-                    seq += 1
+                version = record[0]
+                version = self._version_to_str(id, version)
+                lines.append(version)
             # Next
             id += 1
 
@@ -612,25 +667,33 @@ class Table(File):
     #######################################################################
     # Save (use append for scalability)
     #######################################################################
-    def save_state(self):
+    def _save_state(self):
         if self.incremental_save is False:
-            File.save_state(self)
+            File._save_state(self)
             self.incremental_save = True
             return
 
-        # Incremental Save
-        file = self.safe_open(self.uri, 'a')
+        # Case 1: new file
+        if self.timestamp is None:
+            File._save_state(self)
+            self.changed_properties = False
+            self.added_records = []
+            self.removed_records = []
+            return
+
+        # Case 2: changed file (incremental save)
+        file = self.database.fs.open(self.key, 'a')
         try:
             # Added properties records
-            for seq in self.added_properties:
-                version = self.properties[seq]
-                version = self._version_to_str(-1, seq, version)
+            if self.changed_properties:
+                version = self.properties[0]
+                version = self._version_to_str(-1, version)
                 file.write(version)
-            self.added_properties = []
+            self.changed_properties = False
             # Added records
-            for id, seq in self.added_records:
-                version = self.records[id][seq]
-                version = self._version_to_str(id, seq, version)
+            for id in self.added_records:
+                version = self.records[id][0]
+                version = self._version_to_str(id, version)
                 file.write(version)
             self.added_records = []
             # Removed records
@@ -642,38 +705,16 @@ class Table(File):
         finally:
             file.close()
 
-        # Update the timestamp
-        self.timestamp = vfs.get_mtime(self.uri)
-        self.dirty = None
 
-
-    def save_state_to(self, uri):
+    def save_state_to(self, key):
         # TODO: this is a hack, for 0.50 this case should be covered by the
         # handler's protocol
-        File.save_state_to(self, uri)
-        if uri == self.uri:
+        File.save_state_to(self, key)
+        if key == self.key:
             self.added_records = []
             self.removed_records = []
 
 
-    #######################################################################
-    # API / Private
-    #######################################################################
-    def get_analyser(self, name):
-        datatype = self.get_record_datatype(name)
-        return get_field(datatype.index)
-
-
-    def get_index(self, name):
-        if name not in self.record_schema:
-            raise ValueError, 'the field "%s" is not defined' % name
-
-        if name not in self.catalog.indexes:
-            raise ValueError, 'the field "%s" is not indexed' % name
-
-        return self.catalog.indexes[name]
-
-
     #######################################################################
     # API / Public
     #######################################################################
@@ -689,20 +730,19 @@ class Table(File):
         for name in kw:
             datatype = self.get_record_datatype(name)
             if getattr(datatype, 'unique', False) is True:
-                search = self.search(PhraseQuery(name, kw[name]))
                 if len(self.search(PhraseQuery(name, kw[name]))) > 0:
                     raise UniqueError(name, kw[name])
         # Add version to record
         id = len(self.records)
-        record = self.record_class(id)
-        version = self.properties_to_dict(kw)
+        record = self.record_class(id, self.record_properties)
+        version = record[0]
+        self.properties_to_dict(kw, version)
         version['ts'] = Property(datetime.now())
-        record.append(version)
         # Change
         self.set_changed()
-        self.added_records.append((id, 0))
+        self.added_records.append(id)
         self.records.append(record)
-        self.catalog.index_document(record, id)
+        self.catalog.index_document(record)
         # Back
         return record
 
@@ -720,16 +760,15 @@ class Table(File):
                 if search and (search[0] != self.records[id]):
                     raise UniqueError(name, kw[name])
         # Version of record
-        version = record[-1].copy()
-        version = self.properties_to_dict(kw, version)
+        version = record[0]
+        self.properties_to_dict(kw, version)
         version['ts'] = Property(datetime.now())
         # Change
         self.set_changed()
-        self.catalog.unindex_document(record, id)
-        self.added_records.append((id, len(record)))
-        record.append(version)
+        self.catalog.unindex_document(record.id)
+        self.added_records.append(id)
         # Index
-        self.catalog.index_document(record, id)
+        self.catalog.index_document(record)
 
 
     def update_properties(self, **kw):
@@ -737,18 +776,15 @@ class Table(File):
         if record is None:
             # if the record doesn't exist
             # we create it, it's useful during an update
-            record = self.record_class(-1)
-            version = None
+            record = self.record_class(-1, self.record_properties)
             self.properties = record
-        else:
-            # Version of record
-            version = record[-1].copy()
-        version = self.properties_to_dict(kw, version, first=True)
+
+        version = record[0]
+        self.properties_to_dict(kw, version, first=True)
         version['ts'] = Property(datetime.now())
         # Change
         self.set_changed()
-        self.added_properties.append(len(record))
-        record.append(version)
+        self.changed_properties = True
 
 
     def del_record(self, id):
@@ -758,11 +794,10 @@ class Table(File):
             raise LookupError, msg % id
         # Change
         self.set_changed()
-        if (id, 0) not in self.added_records:
+        if id not in self.added_records:
             self.removed_records.append((id, datetime.now()))
-        self.added_records = [
-            (x, y) for x, y in self.added_records if x != id ]
-        self.catalog.unindex_document(record, id)
+        self.added_records = [ x for x in self.added_records if x != id ]
+        self.catalog.unindex_document(record.id)
         self.records[id] = None
 
 
@@ -807,7 +842,8 @@ class Table(File):
                 return datatype.get_default()
             # Language negotiation ('select_language' is a built-in)
             if language is None:
-                languages = [ x.parameters['language'] for x in property ]
+                languages = [ x.parameters['language'] for x in property
+                              if not datatype.is_empty(x.value) ]
                 language = select_language(languages)
                 if language is None and languages:
                     # Pick up one at random (FIXME)
@@ -854,7 +890,7 @@ class Table(File):
         try:
             return getattr(record, name)
         except AttributeError:
-            if name in self.record_schema:
+            if name in self.record_properties:
                 datatype = self.get_datatype(name)
                 if getattr(datatype, 'multiple', False) is True:
                     return []
@@ -865,21 +901,8 @@ class Table(File):
     def search(self, query=None, **kw):
         """Return list of row numbers returned by executing the query.
         """
-        if query is None:
-            if kw:
-                atoms = []
-                for key, value in kw.items():
-                    atoms.append(PhraseQuery(key, value))
-
-                query = AndQuery(*atoms)
-            else:
-                return self.get_records()
-
-        documents = query.search(self)
-        # Sort by weight
-        ids = documents.keys()
-        ids.sort()
-
+        result = self.catalog.search(query, **kw)
+        ids = [ doc.__id__ for doc in result.get_documents(sort_by='__id__') ]
         return [ self.records[x] for x in ids ]
 
 
@@ -891,11 +914,11 @@ class Table(File):
         - 'columns': the CSV columns used for the mapping between the CSV
           columns and the table schema.
         """
-        record_schema = self.record_schema
-        for line in parse(data, columns, record_schema):
+        record_properties = self.record_properties
+        for line in parse(data, columns, record_properties):
             record = {}
             for index, key in enumerate(columns):
-                if key in record_schema:
+                if key in record_properties:
                     record[key] = line[index]
             self.add_record(record)
 
