@@ -16,7 +16,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from itools
-from base import DataType, is_datatype, copy_datatype
+from base import DataType
 from primitive import Boolean, Decimal, Email, Integer, String, Unicode
 from primitive import Tokens, MultiLinesTokens, Enumerate
 from primitive import PathDataType, URI
@@ -29,10 +29,8 @@ from datetime_ import ISODateTime as DateTime
 
 
 __all__ = [
-    # Abstract clases & helper functions
+    # Abstract class
     'DataType',
-    'is_datatype',
-    'copy_datatype',
     # DataTypes
     'Integer',
     'Decimal',
@@ -53,7 +51,7 @@ __all__ = [
     'ISODateTime',
     'HTTPDate',
     'LanguageTag',
-    # Alias
+    # Aliases
     'Date',
     'Time',
     'DateTime']
