@@ -14,48 +14,24 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-# Import from the Standard Library
-from warnings import warn
-
-
-def is_datatype(datatype, base_type):
-    # TODO To be removed by 0.60
-    warn("The 'is_datatype' function is deprecated.")
-    return issubclass(datatype, base_type)
-
-
-def copy_datatype(datatype, **kw):
-    # TODO To be removed by 0.60
-    warn("The 'copy_datatype' function is deprecated.")
-    return datatype(**kw)
+# Import from itools
+from itools.core import thingy
 
 
 
-class DataType(object):
+class DataType(thingy):
 
     # Default value
     default = None
     multiple = False
 
-    # I18n part
-    context = None
 
-
-    def __new__(cls, **kw):
-        class_name = "%s(%s)" % (cls.__name__, kw)
-        new_datatype = type(class_name, (cls,), kw)
-        for key in kw:
-            setattr(new_datatype, key, kw[key])
-        return new_datatype
-
-
-    @classmethod
     def get_default(cls):
         default = cls.default
         if cls.multiple:
             if isinstance(default, list):
                 return list(default)
-            # Change "default" explicitely to have an initialized list
+            # Change "default" explicitly to have an initialized list
             return []
         return default
 
