@@ -17,22 +17,28 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from itools
+from itools.core import add_type, get_abspath
 from domains import register_domain, get_domain, MSG
-from itools.utils import add_type
 from mo import MOFile
-from po import POFile, POUnit
+from po import POFile, POUnit, encode_source
 
 
 __all__ = [
+    'encode_source',
     'register_domain',
     'get_domain',
     'MSG',
     'MOFile',
     'POFile',
-    'POUnit']
+    'POUnit',
+    ]
 
 
 add_type('text/x-gettext-translation', '.po')
 add_type('text/x-gettext-translation-template', '.pot')
 add_type('application/x-gettext-translation', '.mo')
 add_type('application/x-gettext-translation', '.gmo')
+
+# Register the itools domain
+path = get_abspath('../locale')
+register_domain('itools', path)
