@@ -32,8 +32,7 @@ class MOFile(File):
 
 
     def gettext(self, message):
-        """
-        Returns the translation for the given message.
+        """Returns the translation for the given message.
         """
         return self.translations.ugettext(message)
 
