@@ -22,10 +22,9 @@ from re import compile
 from time import gmtime, strftime, time
 
 # Import from itools
-from itools.handlers import File, TextFile, register_handler_class
+from itools.core import freeze
+from itools.handlers import TextFile, register_handler_class
 from itools.srx import TEXT, START_FORMAT, END_FORMAT
-from itools.utils import freeze
-from itools.xml import XMLParser, TEXT as xml_TEXT, START_ELEMENT, END_ELEMENT
 
 
 ###########################################################################
@@ -242,6 +241,8 @@ class POUnit(object):
     XXX
     """
 
+    __hash__ = None
+
     def __init__(self, comments, context, source, target,
                  references=freeze({}), fuzzy=False):
         self.comments = comments
@@ -556,7 +557,7 @@ class POFile(TextFile):
         return self.messages.keys()
 
 
-    def get_units(self, srx_handler=None):
+    def get_units(self):
         """Returns all the message (objects of the class <POUnit>).
         """
         return self.messages.values()
