@@ -15,15 +15,16 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from itools
-from git import (is_available, get_filenames, get_metadata, get_branch_name,
-    get_tag_names)
+from git import describe, get_branch_name, get_filenames, get_metadata
+from git import get_revisions, is_available
 
 
 
 __all__ = [
+    'describe',
     'get_branch_name',
     'get_filenames',
     'get_metadata',
-    'get_tag_names',
+    'get_revisions',
     'is_available',
     ]
