@@ -21,10 +21,9 @@ from cStringIO import StringIO
 from datetime import datetime
 
 # Import from itools
-from itools.uri import get_absolute_reference
-from itools.vfs import vfs
-from registry import register_handler_class
+from itools.fs import vfs
 from base import Handler
+from registry import register_handler_class
 
 
 
@@ -39,7 +38,7 @@ class File(Handler):
        timestamp/dirty => means...
        -------------------------------------
        None/None => not loaded (yet)
-       None/<dt> => new
+       None/<dt> => new, or moved
        <dt>/None => loaded, but not changed
        <dt>/<dt> => loaded, and changed
 
@@ -50,8 +49,10 @@ class File(Handler):
     dirty = None
 
 
-    def __init__(self, ref=None, string=None, **kw):
-        if ref is None:
+    def __init__(self, key=None, string=None, database=None, **kw):
+        if database is not None:
+            self.database = database
+        if key is None:
             self.reset()
             self.dirty = datetime.now()
             if string is not None:
@@ -61,8 +62,7 @@ class File(Handler):
                 # A handler from some input data
                 self.new(**kw)
         else:
-            # Calculate the URI
-            self.uri = get_absolute_reference(ref)
+            self.key = self.database.normalize_key(key)
 
 
     def reset(self):
@@ -74,9 +74,9 @@ class File(Handler):
 
 
     def __getattr__(self, name):
-        # Not attached to a URI or already loaded (should be correctly
+        # Not attached to a key or already loaded (should be correctly
         # initialized)
-        if self.uri is None or self.timestamp is not None:
+        if self.key is None or self.timestamp is not None:
             message = "'%s' object has no attribute '%s'"
             raise AttributeError, message % (self.__class__.__name__, name)
 
@@ -94,19 +94,22 @@ class File(Handler):
 
 
     def load_state(self):
+        fs = self.database.fs
+        file = fs.open(self.key)
         self.reset()
-        # TODO Use "with" once we move to Python 2.5 and "urllib.urlopen"
-        # supports it
-        file = vfs.open(self.uri)
         try:
             self._load_state_from_file(file)
+        except Exception:
+            self._clean_state()
+            raise
         finally:
             file.close()
-        self.timestamp = vfs.get_mtime(self.uri)
+
+        self.timestamp = fs.get_mtime(self.key)
         self.dirty = None
 
 
-    def load_state_from(self, uri):
+    def load_state_from_uri(self, uri):
         file = vfs.open(uri)
         try:
             self.load_state_from_file(file)
@@ -117,7 +120,11 @@ class File(Handler):
     def load_state_from_file(self, file):
         self.set_changed()
         self.reset()
-        self._load_state_from_file(file)
+        try:
+            self._load_state_from_file(file)
+        except Exception:
+            self._clean_state()
+            raise
 
 
     def load_state_from_string(self, string):
@@ -125,23 +132,35 @@ class File(Handler):
         self.load_state_from_file(file)
 
 
-    def save_state(self):
-        file = self.safe_open(self.uri, 'w')
+    def _save_state(self):
+        file = self.database.fs.open(self.key, 'w')
         try:
             self.save_state_to_file(file)
         finally:
             file.close()
-        # Update the timestamp
-        self.timestamp = vfs.get_mtime(self.uri)
-        self.dirty = None
 
 
-    def save_state_to(self, uri):
-        # If there is an empty folder in the given URI, remove it
-        if vfs.is_folder(uri) and not vfs.get_names(uri):
-            vfs.remove(uri)
+    def save_state(self):
+        if self.dirty:
+            self._save_state()
+            self.timestamp = self.database.fs.get_mtime(self.key)
+            self.dirty = None
+
+
+    def save_state_to(self, key):
+        database = self.database
+
+        # If there is an empty folder in the given key, remove it
+        fs = database.fs if database is not None else vfs
+        if fs.is_folder(key) and not fs.get_names(key):
+            fs.remove(key)
+
         # Save the file
-        file = self.safe_make_file(uri)
+        if database:
+            key = database.normalize_key(key)
+            file = database.fs.make_file(key)
+        else:
+            file = vfs.make_file(key)
         try:
             self.save_state_to_file(file)
         finally:
@@ -154,11 +173,11 @@ class File(Handler):
         # Write and truncate (calls to "_save_state" must be done with the
         # pointer pointing to the beginning)
         file.write(data)
-        file.truncate()
+        file.truncate(file.tell())
 
 
-    def clone(self, cls=None, exclude=('database', 'uri', 'timestamp',
-                                       'dirty')):
+    clone_exclude = frozenset(['database', 'key', 'timestamp', 'dirty'])
+    def clone(self, cls=None):
         # Define the class to build
         if cls is None:
             cls = self.__class__
@@ -168,14 +187,14 @@ class File(Handler):
 
         # Load first, if needed
         if self.dirty is None:
-            if self.uri is not None and self.timestamp is None:
+            if self.key is not None and self.timestamp is None:
                 self.load_state()
 
         # Copy the state
         copy = object.__new__(cls)
-        exclude = set(exclude)
+        copy.reset()
         for name in self.__dict__:
-            if name not in exclude:
+            if name not in cls.clone_exclude:
                 value = getattr(self, name)
                 value = deepcopy(value)
                 setattr(copy, name, value)
@@ -184,7 +203,7 @@ class File(Handler):
 
 
     def is_outdated(self):
-        if self.uri is None:
+        if self.key is None:
             return False
 
         timestamp = self.timestamp
@@ -192,7 +211,7 @@ class File(Handler):
         if timestamp is None:
             return False
 
-        mtime = vfs.get_mtime(self.uri)
+        mtime = self.database.fs.get_mtime(self.key)
         # If the resource layer does not support mtime... we are...
         if mtime is None:
             return True
@@ -201,40 +220,34 @@ class File(Handler):
 
 
     def set_changed(self):
-        # Not attached to a URI
-        if self.uri is None:
-            return
-        # Not attached to a database
+        key = self.key
+
+        # Invalid handler
+        if key is None and self.dirty is None:
+            raise RuntimeError, 'cannot change an orphaned file handler'
+
+        # Free handler (not attached to a database)
         database = self.database
         if database is None:
             self.dirty = datetime.now()
             return
 
-        # Check the handler is in the cache
-        if self.uri not in database.cache:
-            raise RuntimeError, 'cannot change an orphaned file handler'
-        if database.cache[self.uri] is not self:
-            raise RuntimeError, 'cannot change an orphaned file handler'
+        # Attached
+        database.touch_handler(key, self)
 
-        # Update database state
-        if self.timestamp is None and self.dirty is not None:
-            database.added.add(self.uri)
-        else:
-            self.dirty = datetime.now()
-            database.changed.add(self.uri)
+
+    def _clean_state(self):
+        names = [ x for x in self.__dict__ if x not in ('database', 'key') ]
+        for name in names:
+            delattr(self, name)
 
 
     def abort_changes(self):
-        # Not attached to a URI
-        if self.uri is None:
-            return
-        # Not changed
-        if self.dirty is None:
+        # Not attached to a key or not changed
+        if self.key is None or self.dirty is None:
             return
         # Abort
-        names = [ x for x in self.__dict__ if x not in ('database', 'uri') ]
-        for name in names:
-            delattr(self, name)
+        self._clean_state()
 
 
     #########################################################################
@@ -251,8 +264,8 @@ class File(Handler):
         if self.timestamp is not None:
             return self.timestamp
 
-        # Not yet loaded, check the VFS
-        return vfs.get_mtime(self.uri)
+        # Not yet loaded, check the FS
+        return self.database.fs.get_mtime(self.key)
 
 
     def to_str(self):
