@@ -16,7 +16,7 @@
 
 # Import from itools
 from itools.handlers import register_handler_class
-from itools.xml import translate
+from itools.xmlfile import translate
 from xhtml import XHTMLFile, stream_to_str_as_html
 from parser import HTMLParser
 
@@ -57,6 +57,13 @@ class HTMLFile(XHTMLFile):
         self.events = list(stream)
 
 
+    def load_state_from_string(self, string):
+        self.set_changed()
+        self.reset()
+        stream = HTMLParser(string)
+        self.events = list(stream)
+
+
     to_str = XHTMLFile.to_html
 
 
