@@ -15,21 +15,31 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from itools
+from cookies import Cookie, SetCookieDataType
+from entities import Entity
 from exceptions import HTTPError, ClientError, ServerError
 from exceptions import NotModified
 from exceptions import BadRequest, Unauthorized, Forbidden, NotFound
 from exceptions import InternalServerError, NotImplemented, BadGateway
-from exceptions import ServiceUnavailable, MethodNotAllowed
-from request import Request
-from response import Response
-import vfs
+from exceptions import ServiceUnavailable, MethodNotAllowed, Conflict
+from headers import get_type
+from server import HTTPServer
+from utils import set_response
 
 
 __all__ = [
+    'HTTPServer',
+    'Entity',
+    'get_type',
+    'set_response',
+    # Cookies
+    'Cookie',
+    'SetCookieDataType',
     # Exceptions
     'BadGateway',
     'BadRequest',
     'ClientError',
+    'Conflict',
     'Forbidden',
     'HTTPError',
     'InternalServerError',
@@ -40,6 +50,4 @@ __all__ = [
     'ServerError',
     'ServiceUnavailable',
     'Unauthorized',
-    # Classes
-    'Request',
-    'Response']
+    ]
