@@ -15,15 +15,13 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-# Import from the Standard Library
-from base64 import decodestring, encodestring
-
 # Import from itools
 from itools.datatypes import DataType, Integer, String, URI, HTTPDate
 from itools.i18n import AcceptLanguageType
 from cookies import CookieDataType, SetCookieDataType
-from parsing import (read_media_type, read_parameters, read_token,
-    read_white_space)
+from parsing import read_media_type, read_parameters, read_token
+from parsing import read_white_space, UNEXPECTED_CHAR
+
 
 """
 Implementation of standard HTTP headers.
@@ -76,38 +74,6 @@ class ContentType(DataType):
 
 
 
-class Authorization(DataType):
-
-    # While not strictly authorized by RFC 2617, some clients (like
-    # http://validator.w3.org/checklink) send an 'Authorization' header
-    # with an empty value.  So we support this.
-
-    @staticmethod
-    def decode(data):
-        data = data.lstrip()
-        if data == '':
-            return None, None
-        if data.startswith('Basic '):
-            b64auth = data[len('Basic '):]
-            username, password = decodestring(b64auth).split(':', 1)
-            return 'basic', (username, password)
-        raise NotImplementedError, 'XXX'
-
-
-    @staticmethod
-    def encode(value):
-        method, value = value
-        if method is None and value is None:
-            return ''
-        if method == 'basic':
-            username, password = value
-            if ':' in username:
-                raise ValueError, 'XXX'
-            return 'Basic %s' % encodestring('%s:%s' % value)
-        raise NotImplementedError, 'XXX'
-
-
-
 ###########################################################################
 # RFC 2183 (Content-Disposition)
 ###########################################################################
@@ -156,7 +122,7 @@ headers = {
     'via': String,
     'warning': String,
     # Request headers (HTTP 1.0)
-    'authorization': Authorization,
+    'authorization': String,
     'from': String,
     'if-modified-since': IfModifiedSince,
     'referer': URI,
@@ -211,4 +177,5 @@ headers = {
 
 
 def get_type(name):
+    name = name.lower()
     return headers.get(name, String)
