@@ -18,8 +18,8 @@
 # Import from itools
 from accept import AcceptLanguageType, get_accept, select_language
 from accept import init_language_selector
-from base import has_language, get_languages, get_language_name
 from fuzzy import get_distance, get_similarity, is_similar, get_most_similar
+from languages import has_language, get_languages, get_language_name
 from locale_ import format_date, format_time, format_datetime
 from oracle import guess_language, is_asian_character, is_punctuation
 
