@@ -1,7 +1,6 @@
 # -*- coding: UTF-8 -*-
-# Copyright (C) 2005 Nicolas Oyez <nicoyez@gmail.com>
-# Copyright (C) 2005, 2007 Nicolas Deram <nicolas@itaapy.com>
-# Copyright (C) 2007 Juan David Ibáñez Palomar <jdavid@itaapy.com>
+# Copyright (C) 2007 Nicolas Deram <nicolas@itaapy.com>
+# Copyright (C) 2007-2008 Juan David Ibáñez Palomar <jdavid@itaapy.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,7 +23,8 @@ resources in standard type iCalendar specified in the RFC 2445
 
 # Import from itools
 from gridlayout import get_grid_data
-from icalendar import icalendar, Record, icalendarTable
+from icalendar import iCalendar
+from table import Record, icalendarTable
 from types import DateTime, Time
 
 __all__ = [
@@ -34,6 +34,6 @@ __all__ = [
     'DateTime',
     'Time',
     # Handlers
-    'icalendar',
+    'iCalendar',
     'Record',
     'icalendarTable']
