@@ -25,31 +25,26 @@ from itools.html import XHTMLFile
 
 
 # Template to display full day events
-template_string = """
-  <td class="color${event/cal}" xmlns="http://www.w3.org/1999/xhtml">
-    <a href="${event/url}" class="${event/STATUS}">${event/SUMMARY}</a>
-  </td>
-"""
-default_template_fd = XHTMLFile()
-default_template_fd.load_state_from_string(template_string)
+default_template_fd = XHTMLFile(string=
+    """
+    <td class="color${event/cal}" xmlns="http://www.w3.org/1999/xhtml">
+      <a href="${event/url}" class="${event/STATUS}">${event/SUMMARY}</a>
+    </td>""")
 
 # Template to display events with timetables
-template_string = """
-  <td xmlns="http://www.w3.org/1999/xhtml"
+default_template = XHTMLFile(string=
+    """
+    <td xmlns="http://www.w3.org/1999/xhtml"
       xmlns:stl="http://www.hforge.org/xml-namespaces/stl"
-    colspan="${cell/colspan}" rowspan="${cell/rowspan}"
-    valign="top" class="color${cell/content/cal}">
+      colspan="${cell/colspan}" rowspan="${cell/rowspan}"
+      valign="top" class="color${cell/content/cal}">
       <a stl:if="cell/newurl" class="add_event" href="${cell/newurl}">
         <img width="16" height="16" src="${add_icon}" />
       </a><br/>
       <a href="${cell/content/url}" class="${cell/content/STATUS}">
         <span class="summary">${cell/content/SUMMARY}</span>
       </a>
-  </td>
-"""
-default_template = XHTMLFile()
-default_template.load_state_from_string(template_string)
-icon_path = '/ui/images/button_add.png'
+    </td>""")
 
 
 def mcm(l):
@@ -329,7 +324,7 @@ def render_namespace(items, times, with_new_url):
     # render_namespace
     ######################################################################
 
-    url = ';edit_event?method=weekly_view&'
+    url = ';add_event?method=weekly_view&'
     ns_rows = []
     for cells in table:
         ns_cells = []
@@ -355,7 +350,7 @@ def render_namespace(items, times, with_new_url):
 
 
 def get_grid_data(data, grid, start_date=None, templates=(None, None),
-                  with_new_url=True):
+                  with_new_url=True, add_icon=None):
     """
     Build final namespace from data and grid to be used in gridlayout
     templates.
@@ -438,7 +433,7 @@ def get_grid_data(data, grid, start_date=None, templates=(None, None),
                         cell['newurl'] = url
                     if cell['new']:
                         cell['ns'] = stl(template, {'cell': cell,
-                                                    'add_icon': icon_path})
+                                                    'add_icon': add_icon})
             current_date = current_date + timedelta(1)
         cols.append(table)
 
