@@ -23,15 +23,14 @@ from datetime import datetime, timedelta
 from operator import itemgetter
 
 # Import from itools
-from itools.csv import MemoryCatalog, Property
-from itools.csv import parse_table
-from itools.datatypes import String, Unicode
+from itools.core import freeze
+from itools.csv import Property, parse_table, deserialize_parameters
+from itools.datatypes import String, Unicode, DateTime
 from itools.handlers import guess_encoding, TextFile
-from itools.utils import freeze
+from itools.xapian import make_catalog, CatalogAware
 from itools.xapian import PhraseQuery, RangeQuery, OrQuery, AndQuery
-from itools.xapian import KeywordField
 from base import BaseCalendar
-from types import data_properties, Time
+from types import record_properties, record_parameters, Time
 
 
 # The smallest possible difference between non-equal timedelta objects.
@@ -48,7 +47,7 @@ from types import data_properties, Time
 resolution = timedelta.resolution
 
 
-class Component(object):
+class Component(CatalogAware):
     """Parses and evaluates a component block.
 
         input :   string values for c_type and uid
@@ -133,6 +132,15 @@ class Component(object):
         self.versions[sequence] = properties
 
 
+    #######################################################################
+    # CatalogAware part
+    def get_catalog_values(self):
+        values = {'__uid__': self.uid}
+        for name in ['type', 'dtstart', 'dtend']:
+            values[name] = self.get_value(name)
+        return values
+
+
     #######################################################################
     # API / Public
     #######################################################################
@@ -192,29 +200,30 @@ class Component(object):
 
 ##          XXX url: url to access edit_event_form on current event
         """
-        properties = self.get_property
+        get_property = self.get_property
         ns = {}
-        ns['SUMMARY'] = properties('SUMMARY').value
-        ns['ORGANIZER'] = properties('ORGANIZER').value
+        ns['SUMMARY'] = get_property('SUMMARY').value
+        ns['ORGANIZER'] = get_property('ORGANIZER').value
 
         ###############################################################
         # Set dtstart and dtend values using '...' for events which
         # appear into more than one cell
-        start = properties('DTSTART')
-        end = properties('DTEND')
-        param = start.parameters
+        start = get_property('DTSTART')
+        end = get_property('DTEND')
+        start_value_type = start.get_parameter('VALUE', 'DATE-TIME')
+
         ns['start'] = Time.encode(start.value.time())
         ns['end'] = Time.encode(end.value.time())
         ns['TIME'] = None
         if grid:
             # Neither a full day event nor a multiple days event
-            if ('VALUE' not in param or 'DATE' not in param['VALUE']) \
-              and start.value.date() == end.value.date():
+            if (start_value_type != 'DATE'
+                and start.value.date() == end.value.date()):
                 ns['TIME'] = '%s - %s' % (ns['start'], ns['end'])
             else:
                 ns['start'] = ns['end'] = None
         elif not out_on:
-            if 'VALUE' not in param or 'DATE' not in param['VALUE']:
+            if start_value_type != 'DATE':
                 value = ''
                 if starts_on:
                     value = ns['start']
@@ -235,7 +244,7 @@ class Component(object):
             ns['STATUS'] = 'cal_conflict'
         else:
             ns['STATUS'] = 'cal_busy'
-            status = properties('STATUS')
+            status = get_property('STATUS')
             if status:
                 ns['STATUS'] = status.value
 
@@ -257,6 +266,7 @@ class Component(object):
 
 
 
+
 class InnerComponent(Component):
 
     def __init__(self, c_inner_type):
@@ -289,22 +299,14 @@ class iCalendar(BaseCalendar, TextFile):
     class_extension = 'ics'
 
 
-    # To override a component property from the spec, or to add a new one,
-    # define this class variable.
-    schema = {
-##        'DTSTART': DateTime(multiple=False, index='keyword'),
-##        'DTEND': DateTime(multiple=False, index='keyword'),
-    }
+    record_properties = record_properties
+    record_parameters = record_parameters
 
 
     @classmethod
     def get_record_datatype(cls, name):
-        # Overriden schema
-        if name in cls.schema:
-            return cls.schema[name]
-        # The specs schema
-        if name in data_properties:
-            return data_properties[name]
+        if name in cls.record_properties:
+            return cls.record_properties[name]
         # Default
         return String(multiple=True)
 
@@ -312,13 +314,20 @@ class iCalendar(BaseCalendar, TextFile):
     #########################################################################
     # New
     #########################################################################
+    clone_exclude = TextFile.clone_exclude | frozenset(['catalog'])
+
+
     def reset(self):
         self.properties = {}
         self.components = {}
-        self.catalog = MemoryCatalog()
-        self.catalog.add_index('type', KeywordField)
-        self.catalog.add_index('dtstart', KeywordField)
-        self.catalog.add_index('dtend', KeywordField)
+
+        # A Catalog "in memory"
+        fields = {'__uid__': String(is_key_field=True, is_stored=True,
+                                    is_indexed=True),
+                  'type': String(is_indexed=True),
+                  'dtstart': DateTime(is_stored=True, is_indexed=True),
+                  'dtend': DateTime(is_stored=True, is_indexed=True)}
+        self.catalog = make_catalog(None, fields)
 
 
     def new(self):
@@ -351,6 +360,7 @@ class iCalendar(BaseCalendar, TextFile):
             else:
                 value = datatype.decode(value)
             # Build the value (a Property instance)
+            deserialize_parameters(parameters, self.record_parameters)
             value = Property(value, **parameters)
             # Append
             lines.append((name, value))
@@ -358,7 +368,7 @@ class iCalendar(BaseCalendar, TextFile):
         # Read first line
         first = lines[0]
         if (first[0] != 'BEGIN' or first[1].value != 'VCALENDAR'
-            or len(first[1].parameters) != 0):
+            or first[1].parameters):
             raise ValueError, 'icalendar must begin with BEGIN:VCALENDAR'
 
         lines = lines[1:]
@@ -471,7 +481,7 @@ class iCalendar(BaseCalendar, TextFile):
         # Index components
         for uid in self.components:
             component = self.components[uid]
-            self.catalog.index_document(component, uid)
+            self.catalog.index_document(component)
 
 
     #########################################################################
@@ -579,7 +589,7 @@ class iCalendar(BaseCalendar, TextFile):
         component.add_version(kw)
 
         # Index the component
-        self.catalog.index_document(component, uid)
+        self.catalog.index_document(component)
 
         return uid
 
@@ -609,9 +619,9 @@ class iCalendar(BaseCalendar, TextFile):
 
         self.set_changed()
         # Unindex the component, add new version, index again
-        self.catalog.unindex_document(component, uid)
+        self.catalog.unindex_document(uid)
         component.add_version(version)
-        self.catalog.index_document(component, uid)
+        self.catalog.index_document(component)
 
 
     def remove(self, uid):
@@ -623,7 +633,7 @@ class iCalendar(BaseCalendar, TextFile):
         component = self.components[uid]
         del self.components[uid]
         # Unindex
-        self.catalog.unindex_document(component, uid)
+        self.catalog.unindex_document(uid)
 
 
     def get_property_values(self, name=None):
@@ -680,8 +690,8 @@ class iCalendar(BaseCalendar, TextFile):
             events = cal.search_events(
                 STATUS='TENTATIVE',
                 PRIORITY=1,
-                ATTENDEE=[URI.decode('mailto:jdoe@itaapy.com'),
-                          URI.decode('mailto:jsmith@itaapy.com')])
+                ATTENDEE=['mailto:jdoe@itaapy.com',
+                          'mailto:jsmith@itaapy.com'])
 
         ** With a list of values, events match if at least one value matches
 
@@ -694,8 +704,8 @@ class iCalendar(BaseCalendar, TextFile):
         filters = kw.keys()
 
         # For each event
-        events = subset or [self.components[x]
-                            for x in self.search(type='VEVENT')]
+        events = subset or [ self.components[x]
+                             for x in self.search(type='VEVENT') ]
         for event in events:
             version = event.get_version()
 
@@ -740,15 +750,13 @@ class iCalendar(BaseCalendar, TextFile):
         return self.components.get(uid)
 
 
-    def search_events_in_date(self, selected_date, sortby=None, **kw):
+    def search_events_in_date(self, date, sortby=None, **kw):
         """Return a list of Component objects of type 'VEVENT' matching the
         given date and sorted if requested.
         """
-        dtstart = datetime(selected_date.year, selected_date.month,
-                           selected_date.day)
+        dtstart = datetime(date.year, date.month, date.day)
         dtend = dtstart + timedelta(days=1) - resolution
-        return self.search_events_in_range(dtstart, dtend, sortby=sortby,
-                                           **kw)
+        return self.search_events_in_range(dtstart, dtend, sortby=sortby, **kw)
 
 
     def search_events_in_range(self, dtstart, dtend, sortby=None, **kw):
@@ -768,10 +776,8 @@ class iCalendar(BaseCalendar, TextFile):
             dtend = dtend + timedelta(days=1) - resolution
 
         # Get only the events which matches
-        dtstart_limit = str(dtstart + resolution)
-        dtend_limit = str(dtend + resolution)
-        dtstart = str(dtstart)
-        dtend = str(dtend)
+        dtstart_limit = dtstart + resolution
+        dtend_limit = dtend + resolution
         query = AndQuery(
             PhraseQuery('type', 'VEVENT'),
             OrQuery(RangeQuery('dtstart', dtstart, dtend),
@@ -864,38 +870,10 @@ class iCalendar(BaseCalendar, TextFile):
 
     #######################################################################
     # API / Search
-    def get_analyser(self, name):
-        try:
-            return self.catalog.analysers[name]
-        except KeyError:
-            raise ValueError, 'the field "%s" is not indexed' % name
-
-
-    def get_index(self, name):
-        try:
-            return self.catalog.indexes[name]
-        except KeyError:
-            raise ValueError, 'the field "%s" is not indexed' % name
-
-
     def search(self, query=None, **kw):
         """Return list of component internal ids returned by executing the
         query.
         """
-        if query is None:
-            if kw:
-                atoms = []
-                for key, value in kw.items():
-                    atoms.append(PhraseQuery(key, value))
-
-                query = AndQuery(*atoms)
-            else:
-                raise ValueError, "expected a query"
-
-        documents = query.search(self)
-        uids = documents.keys()
-        # Sort by weight
-        uids.sort()
-
-        return uids
+        result = self.catalog.search(query, **kw)
+        return [ x.__uid__ for x in result.get_documents(sort_by='__uid__') ]
 
