@@ -23,13 +23,12 @@ from datetime import datetime, timedelta
 from operator import itemgetter
 
 # Import from itools
+from itools.core import freeze, merge_dicts
 from itools.csv import parse_table, Property, Record as TableRecord, Table
 from itools.datatypes import Integer, String, Unicode
-from itools.handlers import merge_dicts
-from itools.utils import freeze
 from itools.xapian import PhraseQuery, RangeQuery, OrQuery, AndQuery
 from base import BaseCalendar
-from types import data_properties, Time
+from types import record_properties, Time
 
 
 # The smallest possible difference between non-equal timedelta objects.
@@ -51,17 +50,6 @@ class Record(TableRecord):
     """A Record with some icalendar specific methods in addition.
     """
 
-    def get_version(self, sequence=None):
-        """Return the last version of current component or the sequence's one.
-        """
-        if sequence is None:
-            sequence = self[-1]
-        for version in self:
-            if self.get_property('SEQUENCE') == sequence:
-                return version
-        return None
-
-
     # Get a property of current component
     def get_property(self, name=None):
         """Return the value of given name property as a Property or as a list
@@ -107,12 +95,12 @@ class Record(TableRecord):
 
 ##          XXX url: url to access edit_event_form on current event
         """
-        properties = self.get_property
+        get_property = self.get_property
 
-        summary = properties('SUMMARY')
+        summary = get_property('SUMMARY')
         if summary:
             summary = summary.value
-        organizer = properties('ORGANIZER')
+        organizer = get_property('ORGANIZER')
         if organizer:
             organizer = organizer.value
 
@@ -123,21 +111,22 @@ class Record(TableRecord):
         ###############################################################
         # Set dtstart and dtend values using '...' for events which
         # appear into more than one cell
-        start = properties('DTSTART')
-        end = properties('DTEND')
-        param = start.parameters
+        start = get_property('DTSTART')
+        end = get_property('DTEND')
+        start_value_type = start.get_parameter('VALUE', 'DATE-TIME')
+
         ns['start'] = Time.encode(start.value.time())
         ns['end'] = Time.encode(end.value.time())
         ns['TIME'] = None
         if grid:
             # Neither a full day event nor a multiple days event
-            if ('VALUE' not in param or 'DATE' not in param['VALUE']) \
-              and start.value.date() == end.value.date():
+            if (start_value_type != 'DATE'
+                and start.value.date() == end.value.date()):
                 ns['TIME'] = '%s - %s' % (ns['start'], ns['end'])
             else:
                 ns['start'] = ns['end'] = None
         elif not out_on:
-            if 'VALUE' not in param or 'DATE' not in param['VALUE']:
+            if start_value_type != 'DATE':
                 value = ''
                 if starts_on:
                     value = ns['start']
@@ -158,7 +147,7 @@ class Record(TableRecord):
             ns['STATUS'] = 'cal_conflict'
         else:
             ns['STATUS'] = ''
-            status = properties('STATUS')
+            status = get_property('STATUS')
             if status:
                 ns['STATUS'] = status.value
 
@@ -187,21 +176,10 @@ class icalendarTable(BaseCalendar, Table):
 
     record_class = Record
 
-    record_schema = merge_dicts(
-        data_properties,
-        type=String(index='keyword'),
-        inner=Integer(multiple=True),
-    )
-
-
-    def get_parameter_datatype(self, name):
-        """ Default parameter type is multiple for ical.
-        """
-        # Record schema
-        schema = self.parameters_schema
-        if name in schema:
-            return schema[name]
-        return String(multiple=True)
+    record_properties = merge_dicts(
+        record_properties,
+        type=String(is_indexed=True),
+        inner=Integer(multiple=True))
 
 
     #########################################################################
@@ -231,7 +209,7 @@ class icalendarTable(BaseCalendar, Table):
         # Read first line
         first = lines[0]
         if (first[0] != 'BEGIN' or first[1].value != 'VCALENDAR'
-            or len(first[1].parameters) != 0):
+            or first[1].parameters):
             raise ValueError, 'icalendar must begin with BEGIN:VCALENDAR'
 
         lines = lines[1:]
@@ -256,6 +234,7 @@ class icalendarTable(BaseCalendar, Table):
         c_inner_type = None
         uid = None
         records = self.records
+        record_properties = self.record_properties
         id = 0
         uids = {}
 
@@ -280,7 +259,7 @@ class icalendarTable(BaseCalendar, Table):
                     if uid is None:
                         raise ValueError, 'UID is not present'
 
-                    record = self.get_record(id) or Record(id)
+                    record = self.get_record(id) or Record(id, record_properties)
                     c_properties['type'] = Property(c_type)
                     c_properties['UID'] = Property(uid)
                     sequence = c_properties.get('SEQUENCE', None)
@@ -291,14 +270,14 @@ class icalendarTable(BaseCalendar, Table):
                         c_inner_components = [Property(x)
                                               for x in c_inner_components]
                         c_properties['inner'] = c_inner_components
-                    record.append(c_properties)
+                    record[0] = c_properties
                     if uid in uids:
                         n = uids[uid] + 1
                         uids[uid] = n
                     else:
                         n = 0
                         uids[uid] = 0
-                    self.added_records.append((id, n))
+                    self.added_records.append(id)
                     records.append(record)
 
                     # Next
@@ -309,14 +288,14 @@ class icalendarTable(BaseCalendar, Table):
 
                 # Inner component
                 elif value == c_inner_type:
-                    record = self.get_record(id) or Record(id)
+                    record = self.get_record(id) or Record(id, record_properties)
                     c_inner_properties['type'] = Property(c_inner_type)
                     sequence = c_inner_properties.get('SEQUENCE', None)
                     c_inner_properties['SEQUENCE'] = sequence or Property(0)
                     c_inner_properties['ts'] = Property(datetime.now())
-                    record.append(c_inner_properties)
+                    record[0] = c_inner_properties
                     c_inner_components.append(id)
-                    self.added_records.append((id, 0))
+                    self.added_records.append(id)
                     records.append(record)
                     # Next
                     c_inner_type = None
@@ -337,7 +316,7 @@ class icalendarTable(BaseCalendar, Table):
                             # Check the property has not yet being found
                             if prop_name in c_properties:
                                 raise ValueError, \
-                                      "property '%s' can occur only once" % name
+                                    "property '%s' can occur only once" % name
                             # Set the property
                             c_properties[prop_name] = prop_value
                 else:
@@ -357,7 +336,7 @@ class icalendarTable(BaseCalendar, Table):
         # Index the records
         for record in records:
             if record is not None:
-                self.catalog.index_document(record, record.id)
+                self.catalog.index_document(record)
 
 
     def to_ical(self):
@@ -380,38 +359,36 @@ class icalendarTable(BaseCalendar, Table):
         # Calendar components
         for record in self.records:
             if record is not None:
-                seq = 0
                 c_type = record.type
                 # Ignore some components (like DAYLIGHT, STANDARD, ...)
                 # keeping only VEVENT, VTIMEZONE, V.., and x-name ones
                 if not c_type.startswith('V') and not c_type.startswith('X'):
                     continue
-                for version in record:
-                    line = 'BEGIN:%s\n' % c_type
-                    lines.append(Unicode.encode(line))
-                    line = ''
-                    # Properties
-                    names = version.keys()
-                    names.sort()
-                    for name in names:
-                        if name in ('id', 'ts', 'type'):
-                            continue
-                        elif name == 'DTSTAMP':
-                            value = version['ts']
-                        else:
-                            value = version[name]
-                        if name == 'SEQUENCE':
-                            value.value += seq
-                        # Insert inner components
-                        elif name == 'inner':
-                            line = self.encode_inner_components(name, value)
-                        else:
-                            name = name.upper()
-                            line = self.encode_property(name, value)
-                        lines.extend(line)
-                    line = 'END:%s\n' % c_type
-                    lines.append(Unicode.encode(line))
-                    seq += 1
+                version = record[0]
+                line = 'BEGIN:%s\n' % c_type
+                lines.append(Unicode.encode(line))
+                line = ''
+                # Properties
+                names = version.keys()
+                names.sort()
+                for name in names:
+                    if name in ('id', 'ts', 'type'):
+                        continue
+                    elif name == 'DTSTAMP':
+                        value = version['ts']
+                    else:
+                        value = version[name]
+                    if name == 'SEQUENCE':
+                        pass
+                    # Insert inner components
+                    elif name == 'inner':
+                        line = self.encode_inner_components(name, value)
+                    else:
+                        name = name.upper()
+                        line = self.encode_property(name, value)
+                    lines.extend(line)
+                line = 'END:%s\n' % c_type
+                lines.append(Unicode.encode(line))
 
         line = 'END:VCALENDAR\n'
         lines.append(Unicode.encode(line))
@@ -425,15 +402,15 @@ class icalendarTable(BaseCalendar, Table):
             kw['UID'] = self.generate_uid(type)
 
         id = len(self.records)
-        record = Record(id)
-        version = self.properties_to_dict(kw)
+        record = Record(id, self.record_properties)
+        version = record[0]
+        self.properties_to_dict(kw, version)
         version['ts'] = Property(datetime.now())
-        record.append(version)
         # Change
         self.set_changed()
-        self.added_records.append((id, 0))
+        self.added_records.append(id)
         self.records.append(record)
-        self.catalog.index_document(record, id)
+        self.catalog.index_document(record)
         # Back
         return record
 
@@ -466,8 +443,8 @@ class icalendarTable(BaseCalendar, Table):
             events = cal.search_events(
                 STATUS='TENTATIVE',
                 PRIORITY=1,
-                ATTENDEE=[URI.decode('mailto:jdoe@itaapy.com'),
-                          URI.decode('mailto:jsmith@itaapy.com')])
+                ATTENDEE=['mailto:jdoe@itaapy.com',
+                          'mailto:jsmith@itaapy.com'])
 
         ** With a list of values, events match if at least one value matches
 
@@ -535,10 +512,10 @@ class icalendarTable(BaseCalendar, Table):
             dtend = dtend + timedelta(days=1) - resolution
 
         # Get only the events which matches
-        dtstart_limit = str(dtstart + resolution)
-        dtend_limit = str(dtend + resolution)
-        dtstart = str(dtstart)
-        dtend = str(dtend)
+        dtstart_limit = dtstart + resolution
+        dtend_limit = dtend + resolution
+        dtstart = dtstart
+        dtend = dtend
         query = AndQuery(
             PhraseQuery('type', 'VEVENT'),
             OrQuery(RangeQuery('DTSTART', dtstart, dtend),
@@ -585,12 +562,11 @@ class icalendarTable(BaseCalendar, Table):
         return res
 
 
-    def search_events_in_date(self, selected_date, sortby=None, **kw):
+    def search_events_in_date(self, date, sortby=None, **kw):
         """Return a list of Component objects of type 'VEVENT' matching the
         given date and sorted if requested.
         """
-        dtstart = datetime(selected_date.year, selected_date.month,
-                           selected_date.day)
+        dtstart = datetime(date.year, date.month, date.day)
         dtend = dtstart + timedelta(days=1) - resolution
         return self.search_events_in_range(dtstart, dtend, sortby=sortby,
                                            **kw)
