@@ -19,6 +19,7 @@
 from datetime import datetime
 
 # Import from itools
+from itools.core import freeze
 from itools.datatypes import DataType, Integer, ISOTime , URI, Unicode, String
 
 
@@ -39,11 +40,11 @@ class Time(ISOTime):
 class DateTime(DataType):
     """iCalendar Date format:
 
-    DTSTART:19970714T133000.000000 (Local time)
+    DTSTART:19970714T133000.000000 (local time)
 
     DTSTART:19970714T173000Z (UTC time)
 
-    DTSTART;TZID=US-Eastern:19970714T133000 (Local time and time zone reference)
+    DTSTART;TZID=US-Eastern:19970714T133000 (local time + timezone)
     """
 
     @staticmethod
@@ -103,7 +104,7 @@ class DateTime(DataType):
         year, month, day = int(year), int(month), int(day)
         # If no time
         if not time:
-             return datetime(year, month, day)
+            return datetime(year, month, day)
         # Time
         if time.count(':') == 1:
             hours, minutes = time.split(':')
@@ -130,7 +131,7 @@ class DateTime(DataType):
 
 # data types for each property
 # --> TO VERIFY AND COMPLETE
-data_properties = {
+record_properties = freeze({
   'BEGIN': String(multiple=False),
   'END': String(multiple=False),
   'VERSION': Unicode(multiple=False),
@@ -149,12 +150,12 @@ data_properties = {
   'PRIORITY': Integer(multiple=False),
   'RESOURCES': Unicode(multiple=True),
   'STATUS': Unicode(multiple=False),
-  'SUMMARY': Unicode(multiple=False, index='text', mandatory=True),
+  'SUMMARY': Unicode(multiple=False, is_indexed=True, mandatory=True),
   # Date & Time component properties
   'COMPLETED': DateTime(multiple=False),
-  'DTEND': DateTime(multiple=False, index='keyword'),
+  'DTEND': DateTime(multiple=False, is_stored=True, is_indexed=True),
   'DUE': DateTime(multiple=False),
-  'DTSTART': DateTime(multiple=False, index='keyword'),
+  'DTSTART': DateTime(multiple=False, is_stored=True, is_indexed=True),
   'DURATION': Unicode(multiple=False),
   'FREEBUSY': Unicode(multiple=False),
   'TRANSP': Unicode(multiple=False),
@@ -186,8 +187,17 @@ data_properties = {
   'RECURRENCE-ID': DateTime(multiple=False),
   'RELATED-TO': Unicode(multiple=False),
   'URL': URI(multiple=False),
-  'UID': String(multiple=False, index='keyword')
-}
+  'UID': String(multiple=False, is_indexed=True)
+})
+
+
+
+record_parameters = freeze({
+    'ALTREP': URI(multiple=False),
+    # TODO Finish the list
+})
+
+
 
 ################################################################
 #                         NOT USED ACTUALLY
