@@ -15,35 +15,33 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from itools
-from pdf import PDFFile
-from itools.utils import get_abspath
+from itools.core import get_abspath
+from itools.handlers import ro_database
 from itools.relaxng import RelaxNGFile
 from itools.xml import register_dtd
+from pdf import PDFFile
 
 # There are imports from ReportLab in these imports, so, ...
 try:
-    from rml import rmltopdf, stl_rmltopdf
     from pml import pmltopdf, stl_pmltopdf
 except ImportError:
-    print 'You need to install the package "reportlab" to get RML working.'
+    print 'You need to install the package "reportlab" to get PML working.'
 
     # Not implemented, ...
     def not_implemented(*args, **kw):
         raise NotImplementedError, 'the package "reportlab" must be installed'
-    rmltopdf = not_implemented
-    stl_rmltopdf = not_implemented
     pmltopdf = not_implemented
     stl_pmltopdf = not_implemented
 
 
-__all__ = ['PDFFile', 'rmltopdf', 'stl_rmltopdf',
-                      'pmltopdf', 'stl_pmltopdf']
+__all__ = [
+    'PDFFile',
+    'pmltopdf',
+    'stl_pmltopdf']
 
-# Register "rml.dtd"
-register_dtd(get_abspath('rml.dtd'), uri='rml_1_0.dtd')
-register_dtd(get_abspath('rml.dtd'), uri='rml.dtd')
 
 # Read the Relax NG schema of PML and register its namespace
-rng_file = RelaxNGFile(get_abspath('PML-schema.rng'))
+rng_file = get_abspath('PML-schema.rng')
+rng_file = ro_database.get_handler(rng_file, RelaxNGFile)
 rng_file.auto_register()
 
