@@ -16,7 +16,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from itools
-from itools.utils import freeze
+from itools.core import freeze
 
 # Internal import
 from utils import reportlab_value
@@ -100,7 +100,7 @@ class Div(Flowable):
 
         for margin in ('topMargin', 'bottomMargin', 'leftMargin',
                        'rightMargin'):
-            if self.frame_attrs.has_key(margin):
+            if margin in self.frame_attrs:
                 del self.frame_attrs[margin]
 
         border = self.frame_attrs['showBoundary']
