@@ -21,25 +21,24 @@
 
 # Import from the Standard Library
 from cStringIO import StringIO
+from copy import deepcopy
+from socket import setdefaulttimeout
 from os import close as close_fd
 from types import FileType
-import copy
-import socket
-import tempfile
+from tempfile import mkdtemp, mkstemp
 
 # Import from itools
-from itools import get_abspath
+from itools.core import freeze, get_abspath
 from itools.datatypes import XMLContent
 from itools.stl import set_prefix, stl
-from itools.uri import Path
-from itools.uri.uri import get_cwd
-from itools.utils import freeze
-from itools.vfs import vfs
+from itools.uri import get_reference
+from itools.fs import vfs, lfs
 from itools.xml import XMLParser, START_ELEMENT, END_ELEMENT, TEXT
-from itools.xml import get_end_tag, XMLFile
-import itools.http
+from itools.xml import get_end_tag
+from itools.xmlfile import XMLFile
 
 # Internal import
+from css import CssStylesheet
 from doctemplate import MySimpleDocTemplate, MyDocTemplate
 from flowables import Paragraph, Div
 from style import attribute_style_to_dict
@@ -63,9 +62,6 @@ import reportlab
 from reportlab.pdfbase import pdfmetrics
 from reportlab.pdfbase.cidfonts import UnicodeCIDFont
 
-# Import the graphication css parser
-import css
-
 
 ######################################################################
 # Initialization
@@ -126,8 +122,8 @@ def pmltopdf(document, path=None):
     events = XMLParser(data, {None: pml_uri})
 
     if path:
-        here = get_cwd().path
-        prefix = here.resolve2(Path(path))
+        uri = vfs.get_uri(path)
+        prefix = get_reference(uri).path
         stream = set_prefix(events, prefix, ns_uri=pml_uri)
 
     return make_pdf(events)
@@ -142,8 +138,8 @@ def stl_pmltopdf(document, namespace=freeze({}), path=None, mode='pdf'):
     if isinstance(document, XMLFile):
         events = document.events
         # Try to get the path from the file
-        if path is None and document.uri:
-            path = str(document.uri.path)
+        if path is None and document.key:
+            path = document.database.fs.get_path(document.key)
     else:
         events = document
 
@@ -151,8 +147,8 @@ def stl_pmltopdf(document, namespace=freeze({}), path=None, mode='pdf'):
         events = stl(events=events, namespace=namespace)
 
     if path:
-        here = get_cwd().path
-        prefix = here.resolve2(Path(path))
+        uri = vfs.get_uri(path)
+        prefix = get_reference(uri).path
         events = set_prefix(events, prefix, ns_uri=pml_uri)
 
     if mode == 'events':
@@ -186,8 +182,8 @@ def pmltopdf_test(document, path=None):
     events = XMLParser(data, {None: pml_uri})
 
     if path:
-        here = get_cwd().path
-        prefix = here.resolve2(Path(path))
+        uri = vfs.get_uri(path)
+        prefix = get_reference(uri).path
         events = set_prefix(events, prefix, ns_uri=pml_uri)
 
     return document_stream(events, StringIO(), True)
@@ -202,8 +198,8 @@ def stl_pmltopdf_test(document, namespace=freeze({}), path=None):
     if isinstance(document, XMLFile):
         events = document.events
         # Try to get the path from the file
-        if path is None:
-            path = str(document.uri.path)
+        if path is None and document.key:
+            path = document.database.fs.get_path(document.key)
     else:
         events = document
 
@@ -211,8 +207,8 @@ def stl_pmltopdf_test(document, namespace=freeze({}), path=None):
         events = stl(events=events, namespace=namespace)
 
     if path:
-        here = get_cwd().path
-        prefix = here.resolve2(Path(path))
+        uri = vfs.get_uri(path)
+        prefix = get_reference(uri).path
         events = set_prefix(events, prefix, ns_uri=pml_uri)
 
     return document_stream(events, StringIO(), True)
@@ -233,7 +229,7 @@ class Context(object):
 
 
     def __init__(self):
-        self.tmp_dir = tempfile.mkdtemp()
+        self.tmp_dir = mkdtemp()
         self.init_base_style_sheet()
         self.image_not_found_path = get_abspath('missing.png')
         self.toc_place = None
@@ -244,7 +240,7 @@ class Context(object):
         css_file = get_abspath('html.css')
         self.add_css_from_file(css_file)
         self.anchor = []
-        socket.setdefaulttimeout(10)
+        setdefaulttimeout(10)
         self.list_anchor = []
         self.tag_stack = []
         self.style_tag_stack = []
@@ -268,22 +264,16 @@ class Context(object):
 
     def init_base_style_sheet(self):
         self.stylesheet = getSampleStyleSheet()
-        # Add heading level 4, 5 and 6 like in html
-        self.stylesheet.add(ParagraphStyle(name='Heading4',
-                                           parent=self.stylesheet['h3'],
-                                           fontSize=11),
-                            alias='h4')
-        self.stylesheet.add(ParagraphStyle(name='Heading5',
-                                           parent=self.stylesheet['h4'],
-                                           fontSize=10),
-                            alias='h5')
-        self.stylesheet.add(ParagraphStyle(name='Heading6',
-                                           parent=self.stylesheet['h5'],
-                                           fontSize=9),
-                            alias='h6')
-        self.stylesheet.add(ParagraphStyle(name='toctitle',
-                                           parent=self.stylesheet['Normal'],
-                                           fontSize=40))
+        if float(reportlab.Version) < 2.4:
+            # Add heading level 4, 5 and 6 like in html
+            self.stylesheet.add(ParagraphStyle(name='Heading4',
+                parent=self.stylesheet['h3'], fontSize=11), alias='h4')
+            self.stylesheet.add(ParagraphStyle(name='Heading5',
+                parent=self.stylesheet['h4'], fontSize=10), alias='h5')
+            self.stylesheet.add(ParagraphStyle(name='Heading6',
+                parent=self.stylesheet['h5'], fontSize=9), alias='h6')
+            self.stylesheet.add(ParagraphStyle(name='toctitle',
+                parent=self.stylesheet['Normal'], fontSize=40))
 
 
     def get_base_style_sheet(self):
@@ -291,25 +281,26 @@ class Context(object):
 
 
     def get_style(self, name):
+        """Return the style corresponding to name or the style normal if it
+        does not exist.
         """
-           Return the style corresponding to name or the style normal if it
-           does not exist.
-        """
-
-        if self.stylesheet.has_key(name):
+        # Reportlab 2.3 does not fully support the mapping API (__contains__,
+        # get, etc.), so we use a try/except clause.
+        try:
             return self.stylesheet[name]
-        return self.stylesheet['Normal']
+        except KeyError:
+            return self.stylesheet['Normal']
 
 
     def get_tmp_file(self):
-        fd, filename = tempfile.mkstemp(dir=self.tmp_dir)
+        fd, filename = mkstemp(dir=self.tmp_dir)
         # close fd
         close_fd(fd)
-        return vfs.open(filename, 'w')
+        return lfs.open(filename, 'w')
 
 
     def get_toc_anchor(self, tag_name, content):
-        if not self.toc_high_level > tag_name[1]:
+        if not self.toc_high_level > int(tag_name[1]):
             ref = 'toc_' + str(self.cpt_toc_ref)
             self.cpt_toc_ref += 1
             content = '<a name="' + ref + '" />' + content
@@ -318,9 +309,9 @@ class Context(object):
 
     def add_current_style(self, stylesheet_text):
         if self.css is None:
-            self.css = css.CssStylesheet.from_css(stylesheet_text)
+            self.css = CssStylesheet.from_css(stylesheet_text)
         else:
-            tmp = css.CssStylesheet.from_css(stylesheet_text)
+            tmp = CssStylesheet.from_css(stylesheet_text)
             self.css = self.css.merge(tmp)
 
 
@@ -388,7 +379,7 @@ class Context(object):
 
 
     def get_header_copy(self):
-        return copy.deepcopy(self.header)
+        return deepcopy(self.header)
 
 
     def has_footer(self):
@@ -403,12 +394,12 @@ class Context(object):
 
 
     def get_footer_copy(self):
-        return copy.deepcopy(self.footer)
+        return deepcopy(self.footer)
 
 
     def del_tmp_dir(self):
-        if vfs.exists(self.tmp_dir):
-            vfs.remove(self.tmp_dir)
+        if lfs.exists(self.tmp_dir):
+            lfs.remove(self.tmp_dir)
 
 
 
@@ -589,7 +580,8 @@ def body_stream(stream, _tag_name, _attributes, context):
         if event == START_ELEMENT:
             tag_uri, tag_name, attributes = value
             context.path_on_start_event(tag_name, attributes)
-            if tag_name in ('p', 'pre', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6'):
+            if tag_name in ('h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'p', 'pre',
+                            'xpre'):
                 story.extend(paragraph_stream(stream, tag_name, attributes,
                                               context))
             elif tag_name == 'hr':
@@ -662,7 +654,7 @@ def paragraph_stream(stream, elt_tag_name, elt_attributes, context,
     cpt = 0
     has_content = False
     is_not_paragraph = (elt_tag_name != 'p')
-    is_not_pre = (elt_tag_name != 'pre')
+    is_not_pre = (elt_tag_name not in ('pre', 'xpre'))
     is_footer = (elt_tag_name == 'footer')
 
     story = []
@@ -688,8 +680,8 @@ def paragraph_stream(stream, elt_tag_name, elt_attributes, context,
                 skip = True
                 place = len(story)
                 # TODO ? Merge with body_stream?
-                if tag_name in ('p', 'pre', 'h1', 'h2', 'h3', 'h4', 'h5',
-                                'h6'):
+                if tag_name in ('h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'p', 'pre',
+                                'xpre'):
                     story.extend(paragraph_stream(stream, tag_name,
                                                   attributes, context))
                 elif tag_name == 'hr':
@@ -1104,7 +1096,7 @@ def create_paragraph(context, element, content, style_css = {}):
             start_tags += '<%s>' % tag
         while context.style_tag_stack:
             end_tags += get_end_tag(None, context.style_tag_stack.pop())
-    if element[0] == 'pre':
+    if element[0] in ('pre', 'xpre'):
         content = join_content(content)
         content = start_tags + content + end_tags
         widget = XPreformatted(content, style)
@@ -1241,7 +1233,7 @@ class Table_Content(object):
 
         l = len(self.rowHeights)
         if l:
-            none_list = [ None for x in xrange(l, self.current_y) ]
+            none_list = [ None for x in range(l, self.current_y) ]
             self.rowHeights.extend(none_list)
         else:
             self.rowHeights = None
@@ -1275,7 +1267,7 @@ class Table_Content(object):
                 start, stop = self.span_stack.pop()
                 self.add_span(start, stop)
             l = len(self.colWidths)
-            none_list = [ None for x in xrange(l, self.current_x) ]
+            none_list = [ None for x in range(l, self.current_x) ]
             self.colWidths.extend(none_list)
         # Next line
         self.current_x = 0
@@ -1353,7 +1345,7 @@ class Table_Content(object):
             if ctmp > 0:
                 self.span_stack.append(((self.current_x, self.current_y),
                                         (col, row)))
-                self.content[0].extend([ None for x in xrange(ctmp) ])
+                self.content[0].extend([ None for x in range(ctmp) ])
                 self.size[0] += ctmp
                 self.current_x += ctmp - 1
                 return (col, row)
@@ -1370,7 +1362,7 @@ class Table_Content(object):
         list_length = len(self.colWidths)
         platypus_value = format_size(value)
         if not self.current_y and list_length <= self.current_x:
-            none_list = [ None for x in xrange(list_length, self.current_x+1) ]
+            none_list = [ None for x in range(list_length, self.current_x+1) ]
             self.colWidths.extend(none_list)
 
         # Colspan one the first cell
@@ -1387,7 +1379,7 @@ class Table_Content(object):
         list_length = len(self.rowHeights)
         platypus_value = format_size(value)
         if list_length <= self.current_y:
-            none_list = [ None for y in xrange(list_length, self.current_y+1) ]
+            none_list = [ None for y in range(list_length, self.current_y+1) ]
             self.rowHeights.extend(none_list)
         if self.rowHeights[self.current_y] is None\
             or platypus_value > self.rowHeights[self.current_y]:
@@ -1397,7 +1389,7 @@ class Table_Content(object):
     # Internal
     def create_table_line(self):
         line = []
-        line.extend([ 0 for x in xrange(self.size[0]) ])
+        line.extend([ 0 for x in range(self.size[0]) ])
         self.content.append(line)
         self.size[1] += 1
 
@@ -1410,8 +1402,8 @@ class Table_Content(object):
                 st += 1
             else:
                 return
-        for y in xrange(st, stop[1] + 1):
-            for x in xrange(start[0], stop[0] + 1):
+        for y in range(st, stop[1] + 1):
+            for x in range(start[0], stop[0] + 1):
                 if x != start[0] or y != start[1]:
                     self.push_content(None, x, y)
 
