@@ -20,7 +20,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from itools
-from itools.utils import freeze
+from itools.core import freeze
 
 # Internal import
 from utils import font_value, format_size, get_color_as_hexa
@@ -293,6 +293,8 @@ def build_paragraph_style(context, element, style_css):
             style_attr['textColor'] = get_color_as_hexa(value)
         elif key in ('background-color'):
             style_attr['backColor'] = get_color_as_hexa(value)
+        elif key == 'text-indent':
+            style_attr['firstLineIndent'] = format_size(value)
         elif key == 'text-align':
             if value in P_ALIGNMENTS.keys():
                 style_attr['alignment'] = P_ALIGNMENTS.get(value)
@@ -361,7 +363,7 @@ def build_inline_style(context, tag_name, style_css):
             continue
         if tag_and_attrs:
             tag, attrs = tag_and_attrs
-            if style.has_key(tag):
+            if tag in style:
                 style[tag].update(attrs)
             else:
                 style[tag] = attrs
