@@ -15,7 +15,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from itools
-from itools.utils import add_type
+from itools.core import add_type
 from relaxng import RelaxNGFile
 
 __all__ = [
