@@ -16,7 +16,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from itools
-from itools.utils import add_type
+from itools.core import add_type
 from rss import RSSFile
 
 
