@@ -15,10 +15,10 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from itools
-from itools.utils import add_type
+from itools.core import add_type
+from srx import SRXFile
 from segment import Message, get_segments, translate_message
 from segment import TEXT, START_FORMAT, END_FORMAT
-from srx import SRXFile
 
 __all__ = [
     'SRXFile',
