@@ -16,7 +16,8 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from itools
-from itools.utils import get_abspath
+from itools.core import get_abspath
+from itools.handlers import ro_database
 from srx import SRXFile
 
 # Constants
@@ -152,13 +153,13 @@ def _clean_message(message, keep_spaces):
     # (2) Remove start/end couples before and after the message
     while (len(center) >= 2 and center[0][0] == START_FORMAT and
            center[1][0] == END_FORMAT and center[0][1][1] == center[1][1][1]):
-           left.append(center.pop(0))
-           left.append(center.pop(0))
+        left.append(center.pop(0))
+        left.append(center.pop(0))
     while (len(center) >= 2 and center[-2][0] == START_FORMAT and
            center[-1][0] == END_FORMAT and
            center[-2][1][1] == center[-1][1][1]):
-           right.insert(0, center.pop())
-           right.insert(0, center.pop())
+        right.insert(0, center.pop())
+        right.insert(0, center.pop())
 
     # (3) Remove enclosing format
     while center:
@@ -175,6 +176,8 @@ def _clean_message(message, keep_spaces):
     return left, center, right
 
 
+default_srx_handler = get_abspath('srx/default.srx', 'itools')
+default_srx_handler = ro_database.get_handler(default_srx_handler, SRXFile)
 def _split_message(message, srx_handler=None):
     # Concatenation!
     concat_text = []
@@ -185,7 +188,7 @@ def _split_message(message, srx_handler=None):
 
     # Get the rules
     if srx_handler is None:
-        srx_handler = SRXFile(get_abspath('srx/default.srx', 'itools'))
+        srx_handler = default_srx_handler
     # XXX we must handle the language here!
     rules = srx_handler.get_compiled_rules('en')
 
