@@ -15,11 +15,16 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from itools
-from stl import STLError, STLTemplate, stl, set_prefix
+from stl import STLError, STLFile, STLTemplate, stl
+from stl import rewrite_uris, set_prefix
 
 
 __all__ = [
     'STLError',
     'stl',
+    'STLFile',
     'STLTemplate',
-    'set_prefix']
+    # Helper functions
+    'rewrite_uris',
+    'set_prefix',
+    ]
