@@ -23,21 +23,22 @@ language I could imagine.
 """
 
 # Import from the Standard Library
-from copy import copy
+from functools import partial
 from re import compile
 from types import GeneratorType
 
 # Import from itools
-from itools.datatypes import Boolean, URI
+from itools.core import freeze
+from itools.datatypes import Boolean
 from itools.gettext import MSG
-from itools.uri import Path, Reference
-from itools.xml import XMLError, XMLParser, find_end, stream_to_str
-from itools.xml import DOCUMENT_TYPE, START_ELEMENT, END_ELEMENT, TEXT, COMMENT
+from itools.uri import Path, Reference, get_reference
+from itools.xml import XMLParser, find_end, stream_to_str
+from itools.xml import DOCUMENT_TYPE, START_ELEMENT, END_ELEMENT, TEXT
 from itools.xml import xmlns_uri
 from itools.xml import get_attr_datatype
+from itools.xmlfile import XMLFile, get_units
 from itools.html import xhtml_uri
 from itools.html import stream_to_str_as_html, stream_to_str_as_xhtml
-from itools.utils import freeze
 from schema import stl_uri
 
 
@@ -49,7 +50,9 @@ class STLError(StandardError):
     pass
 
 
-ERR_EXPR_VALUE = "unexpected value for the '${%s}' expression"
+ERR_EXPR_NAME = 'name "%s" not found in "${%s}" expression'
+ERR_EXPR_VALUE = 'unexpected value of type "%s" in "${%s}" expression'
+ERR_EXPR_XML = 'expected XML stream not "%s" in "${%s}" expression'
 
 
 
@@ -81,10 +84,14 @@ def evaluate(expression, stack, repeat_stack):
     # Traverse
     value = stack.lookup(path[0])
     for name in path[1:]:
-        value = lookup(value, name)
+        try:
+            value = value[name]
+        except KeyError:
+            raise STLError, ERR_EXPR_NAME % (name, expression)
+
 
     # Call
-    if callable(value):
+    if hasattr(value, '__call__'):
         try:
             value = value()
         except AttributeError, error_value:
@@ -120,23 +127,6 @@ def evaluate_repeat(expression, stack, repeat_stack):
 ###########################################################################
 # Namespace
 ###########################################################################
-def lookup(namespace, name):
-    """Looks for a variable in a namespace (an instance, a mapping, etc..)
-    """
-    # Mapping
-    if isinstance(namespace, dict):
-        if name in namespace:
-            return namespace[name]
-        raise STLError, 'name "%s" not found in the namespace' % name
-
-    # Instance
-    try:
-        return getattr(namespace, name)
-    except AttributeError:
-        raise STLError, 'name "%s" not found in the namespace' % name
-
-
-
 class NamespaceStack(list):
     """This class represents a namespace stack as used by STL. A variable
     is looked up in the stack from the top to the bottom until found.
@@ -147,8 +137,8 @@ class NamespaceStack(list):
         stack.reverse()
         for namespace in stack:
             try:
-                return lookup(namespace, name)
-            except STLError:
+                return namespace[name]
+            except KeyError:
                 pass
 
         raise STLError, 'name "%s" not found in the namespace' % name
@@ -225,51 +215,35 @@ def substitute(data, stack, repeat_stack, encoding='utf-8'):
     Returns a tuple with the interpreted string and the number of
     substitutions done.
     """
-    if not isinstance(data, str):
+    if type(data) is not str:
         raise ValueError, 'byte string expected, not %s' % type(data)
 
-    start = 0
-    state = 0
-    for i, c in enumerate(data):
-        if state == 0:
-            if c == '$':
-                state = 1
-        elif state == 1:
-            if c == '{':
-                end = (i - 1)
-                state = 2
+    segments = subs_expr.split(data)
+    for i, segment in enumerate(segments):
+        if i % 2:
+            # Evaluate expression
+            value = evaluate(segment, stack, repeat_stack)
+            # An STL template
+            if isinstance(value, STLTemplate):
+                value = value.render()
+            # Ignore if None
+            if value is None:
+                continue
+            # Yield
+            if isinstance(value, MSG):
+                value = value.gettext()
+                yield TEXT, value.encode(encoding), 0
+            elif isinstance(value, (list, GeneratorType, XMLParser)):
+                for x in value:
+                    if type(x) is not tuple:
+                        raise STLError, ERR_EXPR_XML % (type(x), segment)
+                    yield x
+            elif type(value) is unicode:
+                yield TEXT, value.encode(encoding), 0
             else:
-                state = 0
-        elif state == 2:
-            if c == '}':
-                # Evaluate expression
-                expression = data[end+2:i]
-                value = evaluate(expression, stack, repeat_stack)
-                # Next
-                if end > start:
-                    yield TEXT, data[start:end], 0
-                start = (i + 1)
-                state = 0
-                # Send back
-                if isinstance(value, STLTemplate):
-                    value = value.render()
-                if value is None:
-                    continue
-                if isinstance(value, MSG):
-                    value = value.gettext()
-                    yield TEXT, value.encode(encoding), 0
-                elif isinstance(value, (list, GeneratorType, XMLParser)):
-                    for x in value:
-                        if type(x) is not tuple:
-                            raise STLError, ERR_EXPR_VALUE % expression
-                        yield x
-                elif type(value) is unicode:
-                    yield TEXT, value.encode(encoding), 0
-                else:
-                    yield TEXT, str(value), 0
-
-    if start <= i:
-        yield TEXT, data[start:], 0
+                yield TEXT, str(value), 0
+        elif segment:
+            yield TEXT, segment, 0
 
 
 
@@ -439,10 +413,35 @@ def process(events, start, end, stack, repeat_stack, encoding, skip_events):
 ########################################################################
 # Set prefix
 ########################################################################
+css_uri_expr = compile (r"url\(([a-zA-Z0-9\./%\-\_]*/%3[bB]{1}download)\);")
+
 def set_prefix(stream, prefix, ns_uri=xhtml_uri):
     if isinstance(prefix, str):
         prefix = Path(prefix)
+    rewrite = partial(resolve_pointer, prefix)
 
+    return rewrite_uris(stream, rewrite, ns_uri)
+
+
+
+def resolve_pointer(offset, value):
+    # FIXME Exception for STL
+    if value[:2] == '${':
+        return value
+
+    # Absolute URI or path
+    uri = get_reference(value)
+    if uri.scheme or uri.authority or uri.path.is_absolute():
+        return value
+
+    # Resolve Path
+    path = offset.resolve(uri.path)
+    value = Reference('', '', path, uri.query.copy(), uri.fragment)
+    return str(value)
+
+
+
+def rewrite_uris(stream, rewrite, ns_uri=xhtml_uri):
     for event in stream:
         type, value, line = event
         # Rewrite URLs (XXX specific to HTML)
@@ -454,39 +453,47 @@ def set_prefix(stream, prefix, ns_uri=xhtml_uri):
                 if tag_uri == ns_uri and attr_uri in (None, ns_uri):
                     # <... src="X" />
                     if attr_name == 'src':
-                        value = resolve_pointer(value, prefix)
+                        value = rewrite(value)
                     # <a href="X"> or <link href="X">
                     elif tag_name in ('a', 'link'):
                         if attr_name == 'href':
-                            value = resolve_pointer(value, prefix)
+                            value = rewrite(value)
+                    elif attr_name == 'style':
+                        # Rewrite url inside style attribute
+                        # Get the chunks
+                        chunks = []
+                        segments = css_uri_expr.split(value)
+                        for index, segment in enumerate(segments):
+                            if index % 2 == 1:
+                                new_segment = rewrite(segment)
+                                chunks.append('url(%s);' % new_segment)
+                            else:
+                                chunks.append(segment)
+                        value = ''.join(chunks)
                     # <param name="movie" value="X" />
                     elif tag_name == 'param':
                         if attr_name == 'value':
                             param_name = attributes.get((attr_uri, 'name'))
                             if param_name == 'movie':
-                                value = resolve_pointer(value, prefix)
+                                value = rewrite(value)
                 aux[(attr_uri, attr_name)] = value
             yield START_ELEMENT, (tag_uri, tag_name, aux), line
         else:
             yield event
 
+###########################################################################
+# STLFile
+###########################################################################
 
+class STLFile(XMLFile):
 
-def resolve_pointer(value, offset):
-    # XXX Exception for STL
-    if value.startswith('${'):
-        return value
+    # FIXME To be changed once we have our own extension and mimetype (#864)
+    class_mimetypes = ['text/xml', 'application/xml', 'application/xhtml+xml']
 
-    uri = URI.decode(value)
-    if not uri.scheme and not uri.authority:
-        if uri.path.is_relative():
-            if uri.path or str(uri) == '.':
-                new_path = offset.resolve(uri.path)
-                value = Reference("", copy(uri.authority), new_path,
-                                  uri.query.copy(), uri.fragment)
-                return str(value)
-
-    return value
+    def get_units(self, srx_handler=None):
+        for source, context, line in get_units(self.events, srx_handler):
+            if len(source) > 1 or subs_expr_solo.match(source[0][1]) is None:
+                yield source, context, line
 
 
 ###########################################################################
