@@ -16,10 +16,11 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from the Standard Library
-from unittest import TestCase, TestLoader, TestSuite, TextTestRunner
+from unittest import TestLoader, TestSuite, TextTestRunner
 
 # Import tests
 import test_abnf
+import test_core
 import test_csv
 import test_datatypes
 import test_gettext
@@ -36,19 +37,19 @@ import test_srx
 import test_stl
 import test_tmx
 import test_uri
-import test_utils # For things not in a sub-package (mostly utils.py)
-import test_vfs
+import test_fs
 import test_web
 import test_workflow
 import test_xliff
 import test_xapian
 import test_xml
+import test_xmlfile
 
-test_modules = [test_abnf, test_csv, test_datatypes, test_gettext,
+test_modules = [test_abnf, test_core, test_csv, test_datatypes, test_gettext,
     test_handlers, test_html, test_http, test_i18n, test_ical, test_odf,
     test_pdf, test_rest, test_rss, test_srx, test_stl, test_tmx, test_uri,
-    test_utils,
-    test_vfs, test_web, test_workflow, test_xliff, test_xapian, test_xml]
+    test_fs, test_web, test_workflow, test_xliff, test_xapian, test_xml,
+    test_xmlfile]
 
 
 loader = TestLoader()
