@@ -17,14 +17,14 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from the Standard Library
-import unittest
-from unittest import TestCase
+from unittest import TestCase, main
 
 # Import from itools
-from itools.datatypes import Boolean, Date, Integer, Unicode, URI, String
 from itools.csv import CSVFile, Table, UniqueError
-from itools.csv.table import unfold_lines
-from itools import vfs
+from itools.csv.table import parse_table, unfold_lines
+from itools.datatypes import Boolean, Date, Integer, Unicode, URI, String
+from itools.fs import lfs
+from itools.handlers import RWDatabase, ro_database
 from itools.xapian import AndQuery, OrQuery, PhraseQuery
 
 
@@ -36,20 +36,22 @@ TEST_DATA_2 = 'one,two,three\nfour,five,six\nseven,eight,nine'
 TEST_SYNTAX_ERROR = '"one",,\n,"two",,\n,,"three"'
 
 
+rw_database = RWDatabase(fs=lfs)
+
 
 class Languages(CSVFile):
 
     columns = ['name', 'url', 'number', 'date']
     schema = {'name': Unicode,
               'url': URI,
-              'number': Integer(index='keyword'),
-              'date': Date(index='keyword')}
+              'number': Integer(is_indexed=True),
+              'date': Date(is_indexed=True)}
 
 
 class Things(CSVFile):
 
     columns = ['name', 'date']
-    schema = {'name': Unicode(index='text'), 'date': Date(index='keyword')}
+    schema = {'name': Unicode(is_indexed=True), 'date': Date(is_indexed=True)}
 
 
 class Numbers(CSVFile):
@@ -62,8 +64,8 @@ class People(CSVFile):
 
     columns = ['name', 'surname', 'date']
     schema = {'name': Unicode,
-              'surname': Unicode(index='text'),
-              'date': Date(index='keyword')}
+              'surname': Unicode(is_indexed=True),
+              'date': Date(is_indexed=True)}
 
 
 class Countries(CSVFile):
@@ -71,15 +73,15 @@ class Countries(CSVFile):
     class_csv_guess = True
     columns = ['id', 'name', 'country', 'date']
     schema = {'id': Integer,
-              'name': Unicode(index='text'),
-              'country': Unicode(index='text'),
-              'date': Date(index='keyword')}
+              'name': Unicode(is_indexed=True),
+              'country': Unicode(is_indexed=True),
+              'date': Date(is_indexed=True)}
 
 
 class Politicians(CSVFile):
 
     columns = ['name', 'is_good']
-    schema = {'name': Unicode, 'is_good': Boolean(index='bool')}
+    schema = {'name': Unicode, 'is_good': Boolean(is_indexed=True)}
 
 
 
@@ -112,9 +114,9 @@ class CSVTestCase(TestCase):
         handler.load_state_from_string(TEST_DATA_1)
         rows = list(handler.get_rows())
         self.assertEqual(rows, [
-            [u"python", URI.decode('http://python.org'), 52343,
+            [u"python", 'http://python.org/', 52343,
              Date.decode('2003-10-23')],
-            [u"ruby", URI.decode('http://ruby-lang.org'), 42352,
+            [u"ruby", 'http://ruby-lang.org/', 42352,
              Date.decode('2001-03-28')]])
 
 
@@ -193,16 +195,16 @@ class CSVTestCase(TestCase):
 
 
     def test_set_state_in_file_resource(self):
-        handler = CSVFile('tests/test.csv')
+        handler = rw_database.get_handler('tests/test.csv', CSVFile)
         handler.add_row(['d1', 'e1', 'f1'])
         handler.save_state()
 
-        handler2 = CSVFile('tests/test.csv')
+        handler2 = rw_database.get_handler('tests/test.csv', CSVFile)
         self.assertEqual(handler2.get_row(3), ['d1', 'e1', 'f1'])
         handler2.del_row(3)
         handler2.save_state()
 
-        handler = CSVFile('tests/test.csv')
+        handler = ro_database.get_handler('tests/test.csv', CSVFile)
         self.assertEqual(handler.get_nrows(), 3)
 
 
@@ -218,8 +220,7 @@ class CSVTestCase(TestCase):
         handler = Things(string=data)
         handler.load_state_from_string(data)
         self.assertEqual(handler.search(date=Date.decode('2005-01-01')), [])
-        self.assertEqual(handler.search(date=Date.decode('2005-10-10')),
-                         [0,2])
+        self.assertEqual(handler.search(date=Date.decode('2005-10-10')), [0,2])
 
 
     def test_index_new_row(self):
@@ -247,10 +248,10 @@ class CSVTestCase(TestCase):
     def test_build_csv_data(self):
         handler = People()
         handler.load_state_from_string('')
-        handler.add_row(['Piotr', 'Macuk', '1975-12-08'])
-        handler.add_row(['Basia', 'Macuk', '2002-02-14'])
+        handler.add_row(['Piotr', 'Macuk', Date.decode('1975-12-08')])
+        handler.add_row(['Basia', 'Macuk', Date.decode('2002-02-14')])
         self.assertEqual(handler.search(surname='macuk'), [0, 1])
-        handler.add_row(['Pawe³', 'Macuk', '1977-05-13'])
+        handler.add_row(['Pawe³', 'Macuk', Date.decode('1977-05-13')])
         self.assertEqual(handler.search(surname='macuk'), [0, 1, 2])
         handler.del_row(2)
         self.assertEqual(handler.search(surname='macuk'), [0, 1])
@@ -260,7 +261,7 @@ class CSVTestCase(TestCase):
 
     def test_advanced_search(self):
         handler = Countries()
-        handler.load_state_from('tests/test_adv.csv')
+        handler.load_state_from_uri('tests/test_adv.csv')
         result1 = handler.search(name='dde', country='sweden')
         self.assertEqual(result1, [5, 6])
 
@@ -329,10 +330,10 @@ email:jacques@itaapy.com
 
 class Agenda(Table):
 
-    record_schema = {
-        'firstname': Unicode(index='text', multiple=False),
+    record_properties = {
+        'firstname': Unicode(is_indexed=True, multiple=False),
         'lastname': Unicode(multiple=False),
-        'email': Unicode(index='keyword', multiple=False, unique=True)}
+        'email': Unicode(is_indexed=True, multiple=False, unique=True)}
 
 
 books_file = """id:0/0
@@ -351,23 +352,16 @@ title;language=es,fr:El Capital
 
 class Books(Table):
 
-    record_schema = {
+    record_properties = {
         'title': Unicode(multiple=True),
         'author': Unicode}
 
-    parameters_schema = {
+    record_parameters = {
         'language': String(multiple=False)}
 
 
 
-class TableTestCase(TestCase):
-
-    def tearDown(self):
-        for name in ['agenda', 'books']:
-            name = 'tests/%s' % name
-            if vfs.exists(name):
-                vfs.remove(name)
-
+class ParsingTableTestCase(TestCase):
 
     def test_unfolding(self):
         """Test unfolding lines."""
@@ -404,6 +398,24 @@ class TableTestCase(TestCase):
             self.assertEqual(line, expected[i])
 
 
+    def test_empty_param_value(self):
+        input = 'a;b=:'
+        lines = parse_table(input)
+        lines = list(lines)
+        self.assertEqual(lines, [('a', '', {'b': ['']})])
+
+
+
+
+class TableTestCase(TestCase):
+
+    def tearDown(self):
+        for name in ['agenda', 'books']:
+            name = 'tests/%s' % name
+            if lfs.exists(name):
+                lfs.remove(name)
+
+
     def test_de_serialize(self):
         data = ('id:0/0\n'
                 'ts:2007-07-13T17:19:21\n'
@@ -435,19 +447,19 @@ class TableTestCase(TestCase):
         agenda = Agenda(string=agenda_file)
         agenda.save_state_to('tests/agenda')
         # Change
-        agenda = Agenda('tests/agenda')
+        agenda = rw_database.get_handler('tests/agenda', Agenda)
         fake = agenda.add_record({'firstname': u'Toto', 'lastname': u'Fofo'})
         agenda.add_record({'firstname': u'Albert', 'lastname': u'Einstein'})
         agenda.del_record(fake.id)
         agenda.save_state()
         # Test
-        agenda = Agenda('tests/agenda')
+        agenda = ro_database.get_handler('tests/agenda', Agenda)
         ids = [ x.id for x in agenda.search(firstname=u'Toto') ]
         self.assertEqual(len(ids), 0)
         ids = [ x.id for x in agenda.search(firstname=u'Albert') ]
         self.assertEqual(len(ids), 1)
         # Clean
-        vfs.remove('tests/agenda')
+        lfs.remove('tests/agenda')
 
 
     def test_unique(self):
@@ -475,7 +487,7 @@ class TableTestCase(TestCase):
         table = Books(string=books_file)
         table.save_state_to('tests/books')
         # Load
-        table = Books('tests/books')
+        table = ro_database.get_handler('tests/books', Books)
         table.load_state()
         # Test
         record_0 = table.get_record(0)
@@ -485,4 +497,4 @@ class TableTestCase(TestCase):
 
 
 if __name__ == '__main__':
-    unittest.main()
+    main()
