@@ -17,11 +17,11 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from the Standard Library
-from time import timezone, mktime
 from datetime import time, date, datetime
-import unittest
-from unittest import TestCase
-import random, decimal
+import decimal
+import random
+from time import timezone, mktime
+from unittest import TestCase, main
 
 # Import from itools
 from itools.datatypes import ISOTime, ISOCalendarDate, ISODateTime, HTTPDate
@@ -54,7 +54,7 @@ class BasicTypeTest(TestCase):
 
 
     def test_Decimal(self):
-        for x in [random.uniform(-100,100) for _ in xrange(10)]:
+        for x in [random.uniform(-100,100) for _ in range(10)]:
             x = decimal.Decimal(str(x))
             data = Decimal.encode(x)
             self.assertEqual(x, Decimal.decode(data))
@@ -126,7 +126,7 @@ class EnumerateTestCase(TestCase):
 
 
     def test_get_value(self):
-        for i in xrange(1,4):
+        for i in range(1,4):
             self.assertEqual(self.AnEnumerate.get_value(
                                 'name%d' % i),
                                 'value%d' % i)
@@ -327,4 +327,4 @@ class LanguageTagTestCase(TestCase):
 
 
 if __name__ == '__main__':
-    unittest.main()
+    main()
