@@ -16,8 +16,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from the Standard Library
-import unittest
-from unittest import TestCase
+from unittest import TestCase, main
 
 # Import from itools
 from itools.gettext import POFile
@@ -102,4 +101,4 @@ class POTestCase(TestCase):
 
 
 if __name__ == '__main__':
-    unittest.main()
+    main()
