@@ -16,30 +16,33 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from the Standard Library
-import unittest
-from unittest import TestCase
+from unittest import TestCase, main
 
 # Import from itools
+from itools.core import start_subprocess
 from itools.csv import Table
 from itools.datatypes import Unicode
-from itools.handlers import get_handler, Database, SafeDatabase, TextFile
-from itools.handlers import ConfigFile
-from itools.uri import get_reference
-from itools import vfs
+from itools.handlers import ro_database
+from itools.handlers import RWDatabase, make_git_database
+from itools.handlers import TextFile, ConfigFile, TGZFile
+from itools.fs import lfs
 
 
+rw_database = RWDatabase(fs=lfs)
+
 
 class Agenda(Table):
 
-    record_schema = {'firstname': Unicode(index='text', multiple=False),
-                     'lastname': Unicode(multiple=False)}
+    record_properties = {
+        'firstname': Unicode(is_indexed=True, multiple=False),
+        'lastname': Unicode(multiple=False)}
 
 
 
 class StateTestCase(TestCase):
 
     def test_abort(self):
-        handler = get_handler('tests/hello.txt')
+        handler = rw_database.get_handler('tests/hello.txt')
         self.assertEqual(handler.data, u'hello world\n')
         handler.set_data(u'bye world\n')
         self.assertEqual(handler.data, u'bye world\n')
@@ -51,12 +54,10 @@ class StateTestCase(TestCase):
 class FolderTestCase(TestCase):
 
     def setUp(self):
-        database = Database()
+        database = RWDatabase(100, 100)
         self.database = database
-        root = get_handler('.')
-        root.database = database
-        self.root = root
-        file = vfs.make_file('tests/toto.txt')
+        self.root = database.get_handler('.')
+        file = lfs.make_file('tests/toto.txt')
         try:
             file.write('I am Toto\n')
         finally:
@@ -64,20 +65,22 @@ class FolderTestCase(TestCase):
 
 
     def tearDown(self):
-        folder = vfs.open('tests')
+        folder = lfs.open('tests')
         for name in 'toto.txt', 'fofo.txt', 'fofo2.txt', 'empty':
             if folder.exists(name):
                 folder.remove(name)
+        if lfs.exists("test_dir"):
+            lfs.remove("test_dir")
 
 
     def test_remove(self):
         folder = self.root.get_handler('tests')
         folder.del_handler('toto.txt')
-        self.assertEqual(vfs.exists('tests/toto.txt'), True)
+        self.assertEqual(lfs.exists('tests/toto.txt'), True)
         self.assertEqual(folder.has_handler('toto.txt'), False)
         # Save
         self.database.save_changes()
-        self.assertEqual(vfs.exists('tests/toto.txt'), False)
+        self.assertEqual(lfs.exists('tests/toto.txt'), False)
         self.assertEqual(folder.has_handler('toto.txt'), False)
 
 
@@ -85,24 +88,35 @@ class FolderTestCase(TestCase):
         folder = self.root.get_handler('tests')
         folder.del_handler('toto.txt')
         folder.set_handler('toto.txt', TextFile())
-        self.assertEqual(vfs.exists('tests/toto.txt'), True)
+        self.assertEqual(lfs.exists('tests/toto.txt'), True)
         self.assertEqual(folder.has_handler('toto.txt'), True)
         # Save
         self.database.save_changes()
-        self.assertEqual(vfs.exists('tests/toto.txt'), True)
+        self.assertEqual(lfs.exists('tests/toto.txt'), True)
         self.assertEqual(folder.has_handler('toto.txt'), True)
 
 
+    def test_add_remove(self):
+        database = self.database
+
+        # Add a new file
+        new_file = TextFile(data=u'New file\n')
+        database.set_handler('test_dir/new_file.txt', new_file)
+
+        # And suppress it
+        database.del_handler('test_dir/new_file.txt')
+
+
     def test_remove_add_remove(self):
         folder = self.root.get_handler('tests')
         folder.del_handler('toto.txt')
         folder.set_handler('toto.txt', TextFile())
         folder.del_handler('toto.txt')
-        self.assertEqual(vfs.exists('tests/toto.txt'), True)
+        self.assertEqual(lfs.exists('tests/toto.txt'), True)
         self.assertEqual(folder.has_handler('toto.txt'), False)
         # Save
         self.database.save_changes()
-        self.assertEqual(vfs.exists('tests/toto.txt'), False)
+        self.assertEqual(lfs.exists('tests/toto.txt'), False)
         self.assertEqual(folder.has_handler('toto.txt'), False)
 
 
@@ -130,7 +144,22 @@ class FolderTestCase(TestCase):
         self.assertEqual(folder.has_handler('toto.txt'), True)
         # Save
         database.save_changes()
-        self.assertEqual(vfs.exists('tests/toto.txt'), True)
+        self.assertEqual(lfs.exists('tests/toto.txt'), True)
+
+
+    def test_remove_folder(self):
+        database = self.database
+
+        # Add a new file
+        new_file = TextFile(data=u'Hello world\n')
+        database.set_handler('test_dir/hello.txt', new_file)
+
+        # Suppress the directory
+        database.del_handler('test_dir')
+
+        # Try to get the file
+        self.assertRaises(LookupError, database.get_handler,
+                          'test_dir/hello.txt')
 
 
     def test_add_abort(self):
@@ -143,7 +172,7 @@ class FolderTestCase(TestCase):
         self.assertEqual(folder.has_handler('fofo.txt'), False)
         # Save
         database.save_changes()
-        self.assertEqual(vfs.exists('tests/fofo.txt'), False)
+        self.assertEqual(lfs.exists('tests/fofo.txt'), False)
 
 
     def test_add_copy(self):
@@ -153,7 +182,7 @@ class FolderTestCase(TestCase):
         folder.copy_handler('fofo.txt', 'fofo2.txt')
         # Save
         database.save_changes()
-        self.assertEqual(vfs.exists('tests/fofo2.txt'), True)
+        self.assertEqual(lfs.exists('tests/fofo2.txt'), True)
 
 
     def test_del_change(self):
@@ -165,30 +194,20 @@ class FolderTestCase(TestCase):
         self.assertRaises(RuntimeError, file.set_data, u'Oh dear\n')
 
 
-    def test_nocache_change(self):
-        """Only can change cached files.
-        """
-        self.database.set_use_cache(False)
-        file = self.root.get_handler('tests/toto.txt')
-        self.database.set_use_cache(True)
-        self.assertRaises(RuntimeError, file.set_data, u'Oh dear\n')
-
-
     def test_empty_folder(self):
         """Empty folders do not exist.
         """
         database = self.database
         root = self.root
         # Setup
-        root.set_handler('tests/empty/toto.txt', TextFile())
+        root.set_handler('tests/empty/sub/toto.txt', TextFile())
         database.save_changes()
-        root.del_handler('tests/empty/toto.txt')
+        root.del_handler('tests/empty/sub/toto.txt')
         database.save_changes()
-        self.assertEqual(vfs.exists('tests/empty'), True)
+        self.assertEqual(lfs.exists('tests/empty'), True)
         # Test
-        root.set_handler('tests/empty', TextFile())
-        database.save_changes()
-        self.assertEqual(vfs.is_file('tests/empty'), True)
+        self.assertRaises(RuntimeError, root.set_handler, 'tests/empty',
+                          TextFile())
 
 
     def test_not_empty_folder(self):
@@ -197,18 +216,30 @@ class FolderTestCase(TestCase):
         database = self.database
         root = self.root
         # Setup
-        root.set_handler('tests/empty/toto.txt', TextFile())
+        root.set_handler('tests/empty/sub/toto.txt', TextFile())
         database.save_changes()
         # Test
         self.assertRaises(RuntimeError, root.set_handler, 'tests/empty',
                           TextFile())
 
 
+    def test_add_get_handlers(self):
+        database = self.database
+
+        # Add a new file
+        new_file = TextFile(data=u'Test get_handlers\n')
+        database.set_handler('test_dir/test_get_handlers.txt', new_file)
+
+        # get_handlers
+        handlers = list(database.get_handlers("test_dir"))
+        self.assertEqual(new_file in handlers, True)
+
+
 
 class TextTestCase(TestCase):
 
     def test_load_file(self):
-        handler = get_handler('tests/hello.txt')
+        handler = ro_database.get_handler('tests/hello.txt')
         self.assertEqual(handler.data, u'hello world\n')
 
 
@@ -218,22 +249,22 @@ class ConfigFileTestCase(TestCase):
 
     def setUp(self):
         self.config_path = "tests/setup.conf.test"
-        if vfs.exists(self.config_path):
-            vfs.remove(self.config_path)
+        if lfs.exists(self.config_path):
+            lfs.remove(self.config_path)
 
 
     def tearDown(self):
-        if vfs.exists(self.config_path):
-            vfs.remove(self.config_path)
+        if lfs.exists(self.config_path):
+            lfs.remove(self.config_path)
 
 
     def _init_test(self, value):
         # Init data
-        if not vfs.exists(self.config_path):
-            vfs.make_file(self.config_path)
+        if not lfs.exists(self.config_path):
+            lfs.make_file(self.config_path)
 
         # Write data
-        config = ConfigFile(ref=self.config_path)
+        config = rw_database.get_handler(self.config_path, ConfigFile)
         config.set_value("test", value)
         config.save_state()
 
@@ -244,9 +275,9 @@ class ConfigFileTestCase(TestCase):
         self._init_test(value)
 
         # Read data
-        config2 = ConfigFile(ref=self.config_path)
+        config2 = rw_database.get_handler(self.config_path, ConfigFile)
         config2_value = config2.get_value("test")
-        vfs.remove(self.config_path)
+        lfs.remove(self.config_path)
 
         # Test data
         self.assertEqual(config2_value, value)
@@ -258,13 +289,13 @@ class ConfigFileTestCase(TestCase):
         self._init_test(value)
 
         # Read data
-        config2 = ConfigFile(ref=self.config_path)
+        config2 = rw_database.get_handler(self.config_path, ConfigFile)
         try:
             config2_value = config2.get_value("test")
         except SyntaxError, e:
             self.fail(e)
         finally:
-            vfs.remove(self.config_path)
+            lfs.remove(self.config_path)
 
         # Test data
         self.assertEqual(config2_value, value)
@@ -276,14 +307,14 @@ class ConfigFileTestCase(TestCase):
         self._init_test(value)
 
         # Write data
-        config = ConfigFile(ref=self.config_path)
+        config = rw_database.get_handler(self.config_path, ConfigFile)
         config.set_value("test", value)
         config.save_state()
 
         # Read data
-        config2 = ConfigFile(ref=self.config_path)
+        config2 = rw_database.get_handler(self.config_path, ConfigFile)
         config2_value = config2.get_value("test")
-        vfs.remove(self.config_path)
+        lfs.remove(self.config_path)
 
         # Test data
         self.assertEqual(config2_value, value)
@@ -295,7 +326,7 @@ class ConfigFileTestCase(TestCase):
         self._init_test(value)
 
         # Write data
-        config = ConfigFile(ref=self.config_path)
+        config = rw_database.get_handler(self.config_path, ConfigFile)
         try:
             config.set_value("test", value)
         except SyntaxError, e:
@@ -303,13 +334,13 @@ class ConfigFileTestCase(TestCase):
         config.save_state()
 
         # Read data
-        config2 = ConfigFile(ref=self.config_path)
+        config2 = rw_database.get_handler(self.config_path, ConfigFile)
         try:
             config2_value = config2.get_value("test")
         except SyntaxError, e:
             self.fail(e)
         finally:
-            vfs.remove(self.config_path)
+            lfs.remove(self.config_path)
 
         # Test data
         self.assertEqual(config2_value, value)
@@ -322,7 +353,7 @@ class ConfigFileTestCase(TestCase):
         self._init_test(value)
 
         # Write data
-        config = ConfigFile(ref=self.config_path)
+        config = rw_database.get_handler(self.config_path, ConfigFile)
         try:
             config.set_value("test", value)
         except SyntaxError, e:
@@ -330,18 +361,31 @@ class ConfigFileTestCase(TestCase):
         config.save_state()
 
         # Read data
-        config2 = ConfigFile(ref=self.config_path)
+        config2 = ro_database.get_handler(self.config_path, ConfigFile)
         try:
             config2_value = config2.get_value("test")
         except SyntaxError, e:
             self.fail(e)
         finally:
-            vfs.remove(self.config_path)
+            lfs.remove(self.config_path)
 
         # Test data
         self.assertEqual(config2_value, value)
 
 
+###########################################################################
+# Archive files
+###########################################################################
+class ArchiveTestCase(TestCase):
+
+    def test_get_handler(self):
+        cls = ro_database.get_handler_class('handlers/test.tar.gz')
+        self.assertEqual(cls, TGZFile)
+
+        file = ro_database.get_handler('handlers/test.tar.gz')
+        self.assertEqual(file.__class__, TGZFile)
+
+
 
 ###########################################################################
 # Safe transactions
@@ -353,20 +397,20 @@ class BrokenHandler(TextFile):
 
 
 
-class DatabaseTestCase(TestCase):
+class GitDatabaseTestCase(TestCase):
 
     def setUp(self):
-        database = SafeDatabase('database.commit')
+        database = make_git_database('fables', 20, 20)
         self.database = database
-        root = get_handler('fables')
-        root.database = database
+        root = database.get_handler('.')
         self.root = root
 
 
     def tearDown(self):
-        for name in ['fables/31.txt', 'fables/agenda', 'database.commit']:
-            if vfs.exists(name):
-                vfs.remove(name)
+        for name in ['fables/31.txt', 'fables/agenda', 'fables/.git',
+                     'fables/broken.txt']:
+            if lfs.exists(name):
+                lfs.remove(name)
 
 
     def test_abort(self):
@@ -378,7 +422,7 @@ class DatabaseTestCase(TestCase):
         # Abort
         self.database.abort_changes()
         # Test
-        self.assertEqual(vfs.exists('fables/31.txt'), False)
+        self.assertEqual(lfs.exists('fables/31.txt'), False)
 
 
     def test_commit(self):
@@ -390,7 +434,7 @@ class DatabaseTestCase(TestCase):
         # Commit
         self.database.save_changes()
         # Test
-        self.assertEqual(vfs.exists('fables/31.txt'), True)
+        self.assertEqual(lfs.exists('fables/31.txt'), True)
 
 
     def test_broken_commit(self):
@@ -405,14 +449,14 @@ class DatabaseTestCase(TestCase):
         # Commit
         self.assertRaises(NameError, self.database.save_changes)
         # Test
-        self.assertEqual(vfs.exists('fables/31.txt'), False)
-        self.assertEqual(vfs.exists('fables/broken.txt'), False)
+        self.assertEqual(lfs.exists('fables/31.txt'), False)
+        self.assertEqual(lfs.exists('fables/broken.txt'), False)
 
 
     def test_append(self):
         database = self.database
         root = self.root
-        # Initalize
+        # Initialize
         agenda = Agenda()
         agenda.add_record({'firstname': u'Karl', 'lastname': u'Marx'})
         agenda.add_record({'firstname': u'Jean-Jacques',
@@ -444,14 +488,19 @@ class DatabaseTestCase(TestCase):
 
         fables.del_handler('31.txt')
         fables.set_handler('31.txt', TextFile())
-        self.assertEqual(vfs.exists('fables/31.txt'), True)
         self.assertEqual(fables.has_handler('31.txt'), True)
         # Save
         self.database.save_changes()
-        self.assertEqual(vfs.exists('fables/31.txt'), True)
+        self.assertEqual(lfs.exists('fables/31.txt'), True)
         self.assertEqual(fables.has_handler('31.txt'), True)
 
 
+    def test_dot_git(self):
+        fables = self.root
+        self.assertRaises(ValueError, fables.del_handler, '.git')
 
+
+
+start_subprocess('fables')
 if __name__ == '__main__':
-    unittest.main()
+    main()
