@@ -16,13 +16,14 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from the Standard Library
-import unittest
 from cStringIO import StringIO
 from datetime import datetime
+from unittest import TestCase, main
 
 # Import from itools
 from itools.csv import Property
-from itools.datatypes import URI
+from itools.csv.table import encode_param_value
+from itools.datatypes import String
 from itools.ical import iCalendar, icalendarTable
 
 
@@ -85,7 +86,23 @@ END:VCALENDAR
 """
 
 
-class icalTestCase(unittest.TestCase):
+def property_to_string(prop_name, prop):
+    """Method only used by test_load and test_load2.
+    """
+    if not prop.parameters:
+        return u'%s:%s' % (prop_name, prop.value)
+
+    params = ''
+    for p_name in prop.parameters:
+        p_value = prop.parameters[p_name]
+        p_value = [ encode_param_value(p_name, x, String) for x in p_value ]
+        param = ';%s=%s' % (p_name, ','.join(p_value))
+        params = params + param
+    return u'%s%s:%s' % (prop_name, params, prop.value)
+
+
+
+class icalTestCase(TestCase):
 
     def setUp(self):
         self.cal1 = iCalendar(string=content)
@@ -93,7 +110,6 @@ class icalTestCase(unittest.TestCase):
 
 
     def test_new(self):
-        """Test new"""
         cal = iCalendar()
 
         properties = []
@@ -105,8 +121,8 @@ class icalTestCase(unittest.TestCase):
 
         # Test properties
         expected_properties = [
-            u'VERSION;{}:2.0',
-            u'PRODID;{}:-//itaapy.com/NONSGML ikaaro icalendar V1.0//EN']
+            u'VERSION;None:2.0',
+            u'PRODID;None:-//itaapy.com/NONSGML ikaaro icalendar V1.0//EN']
         self.assertEqual(properties, expected_properties)
 
         # Test components
@@ -129,7 +145,7 @@ class icalTestCase(unittest.TestCase):
         expected = ['ATTENDEE;MEMBER="mailto:DEV-GROUP@host.com":'
                     'mailto:darwin@itaapy.com\n']
 
-        member = '"mailto:DEV-GROUP@host.com"'
+        member = 'mailto:DEV-GROUP@host.com'
         value = Property('mailto:darwin@itaapy.com', MEMBER=[member])
         output = self.cal1.encode_property('ATTENDEE', value)
         self.assertEqual(output, expected)
@@ -174,8 +190,7 @@ class icalTestCase(unittest.TestCase):
 
 
     def test_add_to_calendar(self):
-        """
-        Test to add property and component to an empty icalendar object.
+        """Test to add property and component to an empty icalendar object.
         """
         cal = iCalendar()
         cal.add_component('VEVENT')
@@ -186,20 +201,9 @@ class icalTestCase(unittest.TestCase):
         self.assertEqual(cal.get_property_values('METHOD'), value)
 
 
-    def property_to_string(self, prop_name, prop):
-        """
-        Method only used by test_load and test_load2.
-        """
-        value, params = prop.value, ''
-        for param_name in prop.parameters:
-            param_value = prop.parameters[param_name]
-            param = u';' + param_name +  u'=' + u','.join(param_value)
-            params = params + param
-        return u'%s%s:%s' % (prop_name, params, value)
-
-
     def test_load(self):
-        """Test loading a simple calendar."""
+        """Test loading a simple calendar.
+        """
         cal = self.cal1
 
         # Test icalendar properties
@@ -215,9 +219,9 @@ class icalTestCase(unittest.TestCase):
             properties.append(property)
 
         expected_properties = [
-            u'VERSION;{}:2.0',
-            u'METHOD;{}:PUBLISH',
-            u'PRODID;{}:-//Mozilla.org/NONSGML Mozilla Calendar V1.0//EN' ]
+            u'VERSION;None:2.0',
+            u'METHOD;None:PUBLISH',
+            u'PRODID;None:-//Mozilla.org/NONSGML Mozilla Calendar V1.0//EN' ]
         self.assertEqual(properties, expected_properties)
 
         # Test component properties
@@ -228,11 +232,11 @@ class icalTestCase(unittest.TestCase):
             datatype = cal.get_record_datatype(prop_name)
             if datatype.multiple is False:
                 prop = version[prop_name]
-                property = self.property_to_string(prop_name, prop)
+                property = property_to_string(prop_name, prop)
                 properties.append(property)
             else:
                 for prop in version[prop_name]:
-                    property = self.property_to_string(prop_name, prop)
+                    property = property_to_string(prop_name, prop)
                     properties.append(property)
 
         expected_event_properties = [
@@ -248,7 +252,7 @@ class icalTestCase(unittest.TestCase):
             u'LOCATION:France',
             u'X-MOZILLA-RECUR-DEFAULT-INTERVAL:0',
             u'DTEND;VALUE=DATE:2005-05-31 00:00:00',
-            u'DTSTART;VALUE="DATE":2005-05-30 00:00:00',
+            u'DTSTART;VALUE=DATE:2005-05-30 00:00:00',
             u'CLASS:PRIVATE']
 
         self.assertEqual(event.uid, '581361a0-1dd2-11b2-9a42-bd3958eeac9a')
@@ -268,7 +272,8 @@ class icalTestCase(unittest.TestCase):
 
 
     def test_load_2(self):
-        """Test loading a 2 events calendar."""
+        """Test loading a 2 events calendar.
+        """
         cal = self.cal2
 
         properties = []
@@ -280,9 +285,9 @@ class icalTestCase(unittest.TestCase):
 
         # Test properties
         expected_properties = [
-            u'VERSION;{}:2.0',
-            u'METHOD;{}:PUBLISH',
-            u'PRODID;{}:-//Mozilla.org/NONSGML Mozilla Calendar V1.0//EN' ]
+            u'VERSION;None:2.0',
+            u'METHOD;None:PUBLISH',
+            u'PRODID;None:-//Mozilla.org/NONSGML Mozilla Calendar V1.0//EN' ]
         self.assertEqual(properties, expected_properties)
 
         events = []
@@ -296,11 +301,11 @@ class icalTestCase(unittest.TestCase):
                 datatype = cal.get_record_datatype(prop_name)
                 if datatype.multiple is False:
                     prop = version[prop_name]
-                    property = self.property_to_string(prop_name, prop)
+                    property = property_to_string(prop_name, prop)
                     properties.append(property)
                 else:
                     for prop in version[prop_name]:
-                        property = self.property_to_string(prop_name, prop)
+                        property = property_to_string(prop_name, prop)
                         properties.append(property)
 
             events.append(properties)
@@ -316,7 +321,7 @@ class icalTestCase(unittest.TestCase):
             u'LOCATION:France',
             u'X-MOZILLA-RECUR-DEFAULT-INTERVAL:0',
             u'DTEND;VALUE=DATE:2005-05-31 23:59:59.999999',
-            u'DTSTART;VALUE="DATE":2005-05-30 00:00:00',
+            u'DTSTART;VALUE=DATE:2005-05-30 00:00:00',
             u'CLASS:PRIVATE'],
             [
             u'ATTENDEE;MEMBER="mailto:DEV-GROUP@host2.com";RSVP=TRUE'\
@@ -324,7 +329,7 @@ class icalTestCase(unittest.TestCase):
             u'SUMMARY:222222222',
             u'PRIORITY:2',
             u'DTEND;VALUE=DATE:2005-07-01 00:00:00',
-            u'DTSTART;VALUE="DATE":2005-07-01 00:00:00'
+            u'DTSTART;VALUE=DATE:2005-07-01 00:00:00'
             ]]
 
         self.assertEqual(events, expected_events)
@@ -344,13 +349,15 @@ class icalTestCase(unittest.TestCase):
 
     # Just call to_str method
     def test_to_str(self):
-        """Call to_str method."""
+        """Call to_str method.
+        """
         cal = self.cal2
         cal.to_str()
 
 
     def test_add_property(self):
-        """ Test adding a property to any component """
+        """Test adding a property to any component.
+        """
         cal = self.cal2
         event = cal.get_components('VEVENT')[1]
 
@@ -384,7 +391,8 @@ class icalTestCase(unittest.TestCase):
 
 
     def test_icalendar_set_property(self):
-        """ Test setting a new value to an existant icalendar property"""
+        """Test setting a new value to an existant icalendar property.
+        """
         cal = self.cal1
 
         name, value = 'VERSION', Property('2.1')
@@ -396,7 +404,8 @@ class icalTestCase(unittest.TestCase):
 
 
     def test_component_set_property(self):
-        """ Test setting a new value to an existant component property"""
+        """Test setting a new value to an existant component property.
+        """
         cal = self.cal1
         event = cal.get_components('VEVENT')[0]
 
@@ -406,18 +415,19 @@ class icalTestCase(unittest.TestCase):
 
         member = '"mailto:DEV-GROUP@host2.com"'
         value = [
-            Property(URI.decode('mailto:darwin@itaapy.com'), MEMBER=[member]),
-            Property(URI.decode('mailto:jdoe@itaapy.com')),
-            Property(URI.decode('mailto:jsmith@itaapy.com'))]
+            Property('mailto:darwin@itaapy.com', MEMBER=[member]),
+            Property('mailto:jdoe@itaapy.com'),
+            Property('mailto:jsmith@itaapy.com')]
         cal.update_component(event.uid, ATTENDEE=value)
         self.assertEqual(event.get_property_values('ATTENDEE'), value)
 
 
     def test_search_events(self):
-        """Test get events filtered by arguments given."""
+        """Test get events filtered by arguments given.
+        """
         # Test with 1 event
         cal = self.cal1
-        attendee_value = URI.decode('mailto:jdoe@itaapy.com')
+        attendee_value = 'mailto:jdoe@itaapy.com'
 
         events = cal.search_events(ATTENDEE=attendee_value)
         self.assertEqual(len(events), 1)
@@ -435,14 +445,14 @@ class icalTestCase(unittest.TestCase):
         self.assertEqual(len(events), 1)
 
         events = cal.search_events(
-            ATTENDEE=[attendee_value, URI.decode('mailto:jsmith@itaapy.com')],
+            ATTENDEE=[attendee_value, 'mailto:jsmith@itaapy.com'],
             STATUS='TENTATIVE',
             PRIORITY=1)
         self.assertEqual(len(events), 1)
 
         # Tests with 2 events
         cal = self.cal2
-        attendee_value = URI.decode('mailto:jdoe@itaapy.com')
+        attendee_value = 'mailto:jdoe@itaapy.com'
 
         events = cal.search_events(ATTENDEE=attendee_value)
         self.assertEqual(len(events), 2)
@@ -460,14 +470,15 @@ class icalTestCase(unittest.TestCase):
         self.assertEqual(len(events), 1)
 
         events = cal.search_events(
-            ATTENDEE=[attendee_value, URI.decode('mailto:jsmith@itaapy.com')],
+            ATTENDEE=[attendee_value, 'mailto:jsmith@itaapy.com'],
             STATUS='TENTATIVE',
             PRIORITY=1)
         self.assertEqual(len(events), 1)
 
 
     def test_search_events_in_date(self):
-        """Test search events by date."""
+        """Test search events by date.
+        """
         cal = self.cal1
 
         date = datetime(2005, 5, 29)
@@ -485,7 +496,7 @@ class icalTestCase(unittest.TestCase):
         events = cal.search_events_in_date(date, STATUS='CONFIRMED')
         self.assertEqual(len(events), 0)
 
-        attendee_value = URI.decode('mailto:jdoe@itaapy.com')
+        attendee_value = 'mailto:jdoe@itaapy.com'
         events = cal.search_events_in_date(date, ATTENDEE=attendee_value)
         self.assertEqual(len(events), 1)
         events = cal.search_events_in_date(date, ATTENDEE=attendee_value,
@@ -502,7 +513,8 @@ class icalTestCase(unittest.TestCase):
 
 
     def test_search_events_in_range(self):
-        """Test search events matching given dates range."""
+        """Test search events matching given dates range.
+        """
         cal = self.cal2
 
         dtstart = datetime(2005, 1, 1)
@@ -545,7 +557,7 @@ class icalTestCase(unittest.TestCase):
         events = cal.search_events_in_range(dtstart, dtend, STATUS='CONFIRMED')
         self.assertEqual(len(events), 0)
 
-        attendee_value = URI.decode('mailto:jdoe@itaapy.com')
+        attendee_value = 'mailto:jdoe@itaapy.com'
         events = cal.search_events_in_range(dtstart, dtend,
                                             ATTENDEE=attendee_value)
         self.assertEqual(len(events), 1)
@@ -558,8 +570,7 @@ class icalTestCase(unittest.TestCase):
 
 
     def test_get_conflicts(self):
-        """
-        Test get_conflicts method which returns uid couples of events
+        """Test get_conflicts method which returns uid couples of events
         conflicting on a given date.
         """
         cal = self.cal2
@@ -579,7 +590,7 @@ class icalTestCase(unittest.TestCase):
 
 
 
-class icalTableTestCase(unittest.TestCase):
+class icalTableTestCase(TestCase):
 
     def setUp(self):
         src = iCalendar(string=content)
@@ -596,7 +607,6 @@ class icalTableTestCase(unittest.TestCase):
 
 
     def test_new(self):
-        """Test new"""
         cal = icalendarTable()
 
         # Test components
@@ -619,7 +629,7 @@ class icalTableTestCase(unittest.TestCase):
         expected = ['ATTENDEE;MEMBER="mailto:DEV-GROUP@host.com":'
                     'mailto:darwin@itaapy.com\n']
 
-        member = '"mailto:DEV-GROUP@host.com"'
+        member = 'mailto:DEV-GROUP@host.com'
         value = Property('mailto:darwin@itaapy.com', MEMBER=[member])
         output = self.cal1.encode_property('ATTENDEE', value)
         self.assertEqual(output, expected)
@@ -660,28 +670,16 @@ class icalTableTestCase(unittest.TestCase):
 
 
     def test_add_to_calendar(self):
-        """
-        Test to add property and component to an empty icalendar object.
+        """Test to add property and component to an empty icalendar object.
         """
         cal = icalendarTable()
         cal.add_record({'type': 'VEVENT'})
         self.assertEqual(len(cal.get_components('VEVENT')), 1)
 
 
-    def property_to_string(self, prop_name, prop):
-        """
-        Method only used by test_load and test_load2.
-        """
-        value, params = prop.value, ''
-        for param_name in prop.parameters:
-            param_value = prop.parameters[param_name]
-            param = u';' + param_name +  u'=' + u','.join(param_value)
-            params = params + param
-        return u'%s%s:%s' % (prop_name, params, value)
-
-
     def test_load(self):
-        """Test loading a simple calendar."""
+        """Test loading a simple calendar.
+        """
         cal = self.cal1
 
         # Test component properties
@@ -694,11 +692,11 @@ class icalTableTestCase(unittest.TestCase):
             datatype = cal.get_record_datatype(prop_name)
             if getattr(datatype, 'multiple', False) is False:
                 prop = version[prop_name]
-                property = self.property_to_string(prop_name, prop)
+                property = property_to_string(prop_name, prop)
                 properties.append(property)
             else:
                 for prop in version[prop_name]:
-                    property = self.property_to_string(prop_name, prop)
+                    property = property_to_string(prop_name, prop)
                     properties.append(property)
 
         expected_event_properties = [
@@ -714,7 +712,7 @@ class icalTableTestCase(unittest.TestCase):
             u'LOCATION:France',
             u'X-MOZILLA-RECUR-DEFAULT-INTERVAL:0',
             u'DTEND;VALUE=DATE:2005-05-31 00:00:00',
-            u'DTSTART;VALUE="DATE":2005-05-30 00:00:00',
+            u'DTSTART;VALUE=DATE:2005-05-30 00:00:00',
             u'CLASS:PRIVATE']
 
         self.assertEqual(event.UID, '581361a0-1dd2-11b2-9a42-bd3958eeac9a')
@@ -734,7 +732,8 @@ class icalTableTestCase(unittest.TestCase):
 
 
     def test_load_2(self):
-        """Test loading a 2 events calendar."""
+        """Test loading a 2 events calendar.
+        """
         cal = self.cal2
 
         events = []
@@ -750,11 +749,11 @@ class icalTableTestCase(unittest.TestCase):
                 datatype = cal.get_record_datatype(prop_name)
                 if getattr(datatype, 'multiple', False) is False:
                     prop = version[prop_name]
-                    property = self.property_to_string(prop_name, prop)
+                    property = property_to_string(prop_name, prop)
                     properties.append(property)
                 else:
                     for prop in version[prop_name]:
-                        property = self.property_to_string(prop_name, prop)
+                        property = property_to_string(prop_name, prop)
                         properties.append(property)
 
             events.append(properties)
@@ -766,7 +765,7 @@ class icalTableTestCase(unittest.TestCase):
             u'SUMMARY:222222222',
             u'PRIORITY:2',
             u'DTEND;VALUE=DATE:2005-07-01 00:00:00',
-            u'DTSTART;VALUE="DATE":2005-07-01 00:00:00'
+            u'DTSTART;VALUE=DATE:2005-07-01 00:00:00'
             ],
             [
             u'STATUS:TENTATIVE',
@@ -778,7 +777,7 @@ class icalTableTestCase(unittest.TestCase):
             u'LOCATION:France',
             u'X-MOZILLA-RECUR-DEFAULT-INTERVAL:0',
             u'DTEND;VALUE=DATE:2005-05-31 23:59:59.999999',
-            u'DTSTART;VALUE="DATE":2005-05-30 00:00:00',
+            u'DTSTART;VALUE=DATE:2005-05-30 00:00:00',
             u'CLASS:PRIVATE']
             ]
 
@@ -799,13 +798,15 @@ class icalTableTestCase(unittest.TestCase):
 
     # Just call to_ical method
     def test_to_ical(self):
-        """Call to_ical method."""
+        """Call to_ical method.
+        """
         cal = self.cal2
         cal.to_ical()
 
 
     def test_add_property(self):
-        """ Test adding a property to any component """
+        """Test adding a property to any component.
+        """
         cal = self.cal2
         event = cal.get_components('VEVENT')[1]
 
@@ -839,7 +840,8 @@ class icalTableTestCase(unittest.TestCase):
 
 
     def test_component_set_property(self):
-        """ Test setting a new value to an existant component property"""
+        """Test setting a new value to an existant component property.
+        """
         cal = self.cal1
         event = cal.get_components('VEVENT')[0]
 
@@ -849,18 +851,19 @@ class icalTableTestCase(unittest.TestCase):
 
         member = '"mailto:DEV-GROUP@host2.com"'
         value = [
-            Property(URI.decode('mailto:darwin@itaapy.com'), MEMBER=[member]),
-            Property(URI.decode('mailto:jdoe@itaapy.com')),
-            Property(URI.decode('mailto:jsmith@itaapy.com'))]
+            Property('mailto:darwin@itaapy.com', MEMBER=[member]),
+            Property('mailto:jdoe@itaapy.com'),
+            Property('mailto:jsmith@itaapy.com')]
         cal.update_record(event.id, ATTENDEE=value)
         self.assertEqual(event.get_property('ATTENDEE'), value)
 
 
     def test_search_events(self):
-        """Test get events filtered by arguments given."""
+        """Test get events filtered by arguments given.
+        """
         cal = self.cal1
         # Test with 1 event
-        attendee_value = URI.decode('mailto:jdoe@itaapy.com')
+        attendee_value = 'mailto:jdoe@itaapy.com'
 
         events = cal.search_events(ATTENDEE=attendee_value)
         self.assertEqual(len(events), 1)
@@ -878,14 +881,14 @@ class icalTableTestCase(unittest.TestCase):
         self.assertEqual(len(events), 1)
 
         events = cal.search_events(
-            ATTENDEE=[attendee_value, URI.decode('mailto:jsmith@itaapy.com')],
+            ATTENDEE=[attendee_value, 'mailto:jsmith@itaapy.com'],
             STATUS='TENTATIVE',
             PRIORITY=1)
         self.assertEqual(len(events), 1)
 
         # Tests with 2 events
         cal = iCalendar(string=content2)
-        attendee_value = URI.decode('mailto:jdoe@itaapy.com')
+        attendee_value = 'mailto:jdoe@itaapy.com'
 
         events = cal.search_events(ATTENDEE=attendee_value)
         self.assertEqual(len(events), 2)
@@ -903,14 +906,15 @@ class icalTableTestCase(unittest.TestCase):
         self.assertEqual(len(events), 1)
 
         events = cal.search_events(
-            ATTENDEE=[attendee_value, URI.decode('mailto:jsmith@itaapy.com')],
+            ATTENDEE=[attendee_value, 'mailto:jsmith@itaapy.com'],
             STATUS='TENTATIVE',
             PRIORITY=1)
         self.assertEqual(len(events), 1)
 
 
     def test_search_events_in_date(self):
-        """Test search events by date."""
+        """Test search events by date.
+        """
         cal = self.cal1
 
         date = datetime(2005, 5, 29)
@@ -928,7 +932,7 @@ class icalTableTestCase(unittest.TestCase):
         events = cal.search_events_in_date(date, STATUS='CONFIRMED')
         self.assertEqual(len(events), 0)
 
-        attendee_value = URI.decode('mailto:jdoe@itaapy.com')
+        attendee_value = 'mailto:jdoe@itaapy.com'
         events = cal.search_events_in_date(date, ATTENDEE=attendee_value)
         self.assertEqual(len(events), 1)
         events = cal.search_events_in_date(date, ATTENDEE=attendee_value,
@@ -945,7 +949,8 @@ class icalTableTestCase(unittest.TestCase):
 
 
     def test_search_events_in_range(self):
-        """Test search events matching given dates range."""
+        """Test search events matching given dates range.
+        """
         cal = self.cal2
 
         dtstart = datetime(2005, 1, 1)
@@ -988,7 +993,7 @@ class icalTableTestCase(unittest.TestCase):
         events = cal.search_events_in_range(dtstart, dtend, STATUS='CONFIRMED')
         self.assertEqual(len(events), 0)
 
-        attendee_value = URI.decode('mailto:jdoe@itaapy.com')
+        attendee_value = 'mailto:jdoe@itaapy.com'
         events = cal.search_events_in_range(dtstart, dtend,
                                             ATTENDEE=attendee_value)
         self.assertEqual(len(events), 1)
@@ -1001,8 +1006,7 @@ class icalTableTestCase(unittest.TestCase):
 
 
     def test_get_conflicts(self):
-        """
-        Test get_conflicts method which returns uid couples of events
+        """Test get_conflicts method which returns uid couples of events
         conflicting on a given date.
         """
         cal = self.cal2
@@ -1022,4 +1026,4 @@ class icalTableTestCase(unittest.TestCase):
 
 
 if __name__ == '__main__':
-    unittest.main()
+    main()
