@@ -16,14 +16,13 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-# Import from Python
-import unittest
-from unittest import TestCase
+# Import from the Standard Library
+from unittest import TestCase, main
 
 # Import from itools
-from itools.uri import Reference
-from itools.xml import XMLParser, XML_DECL
+from itools.handlers import ro_database
 from itools.rss import RSSFile
+from itools.xml import XMLParser, XML_DECL
 
 
 
@@ -33,14 +32,13 @@ class RSSTestCase(TestCase):
     """
 
     def setUp(self):
-        self.rss = RSSFile('tests/sample-rss-2.xml')
+        self.rss = ro_database.get_handler('tests/sample-rss-2.xml', RSSFile)
 
 
     def test_channel(self):
         channel = self.rss.channel
         self.assertEqual(channel['title'], u"Liftoff News")
         link = channel['link']
-        self.assert_(isinstance(link, Reference))
         self.assertEqual(str(link), 'http://liftoff.msfc.nasa.gov/')
         self.assertEqual(channel['description'],
                 u"Liftoff to Space Exploration.")
@@ -60,7 +58,6 @@ class RSSTestCase(TestCase):
         item = self.rss.items[0]
         self.assertEqual(item['title'], u"Star City")
         link = item['link']
-        self.assert_(isinstance(link, Reference))
         self.assertEqual(str(link),
                 'http://liftoff.msfc.nasa.gov/news/2003/news-starcity.asp')
         expected = u"How do Americans get ready to work with Russians"
@@ -85,4 +82,4 @@ class RSSTestCase(TestCase):
 
 
 if __name__ == '__main__':
-    unittest.main()
+    main()
