@@ -15,17 +15,17 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from the Standard Library
-import unittest
+from unittest import TestCase, main
 
 # Import from itools
-from itools.handlers import get_handler
+import itools.html
 from itools.stl import stl
 from itools.stl.stl import NamespaceStack, substitute, evaluate
-from itools.xml import XMLFile, stream_to_str
-import itools.html
+from itools.xml import stream_to_str
+from itools.xmlfile import XMLFile
 
 
-class SubstituteTestCase(unittest.TestCase):
+class SubstituteTestCase(TestCase):
 
     def setUp(self):
         namespace = {'name': u'Toto'}
@@ -44,7 +44,7 @@ class SubstituteTestCase(unittest.TestCase):
 
 
 
-class STLTestCase(unittest.TestCase):
+class STLTestCase(TestCase):
 
     def test_none(self):
         stack = NamespaceStack()
@@ -122,4 +122,4 @@ class STLTestCase(unittest.TestCase):
 
 
 if __name__ == '__main__':
-    unittest.main()
+    main()
