@@ -16,10 +16,10 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from the Standard Library
-import unittest
-from unittest import TestCase
+from unittest import TestCase, main
 
 # Import from itools
+from itools.handlers import ro_database
 from itools.tmx import TMXFile
 
 
@@ -29,10 +29,10 @@ class TMXTestCase(TestCase):
     def test_input(self):
         """Test input.
         """
-        tmx = TMXFile('tests/localizermsgs.tmx')
+        tmx = ro_database.get_handler('tests/localizermsgs.tmx', TMXFile)
         tmx.to_str()
 
 
 
 if __name__ == '__main__':
-    unittest.main()
+    main()
