@@ -17,13 +17,12 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from the Standard Library
-import unittest
-from unittest import TestCase
+from unittest import TestCase, main
 
 # Import from itools
-from itools.uri import get_reference, Path, Reference
+from itools.uri import get_reference, normalize_path, Path, Reference
 from itools.uri.parsing import uri_parser, parse_uri
-from itools.uri.generic import normalize_path, GenericDataType
+from itools.uri.generic import GenericDataType
 from itools.uri.mailto import Mailto, MailtoDataType
 
 
@@ -181,13 +180,13 @@ class PathResolve2TestCase(TestCase):
     def test_resolve2_wo_slash(self):
         before = Path('/a/b')
         after = Path('/a/b/c')
-        self.assertEqual(before.resolve2('c'), after)
+        self.assertEqual(before.resolve_name('c'), after)
 
 
     def test_resolve2_w_slash(self):
         before = Path('/a/b/')
         after = Path('/a/b/c')
-        self.assertEqual(before.resolve2('c'), after)
+        self.assertEqual(before.resolve_name('c'), after)
 
 
 
@@ -500,4 +499,4 @@ class MailtoTestCase(TestCase):
 
 
 if __name__ == '__main__':
-    unittest.main()
+    main()
