@@ -15,13 +15,10 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from the Standard Library
-import unittest
-from unittest import TestCase
+from unittest import TestCase, main
 
 # Import from itools
-import itools.http
-from itools import vfs
-from itools.web import BaseView, RootResource, Server
+from itools.web import BaseView, RootResource, WebServer
 
 
 class MyRootView(BaseView):
@@ -39,11 +36,11 @@ class ServerTestCase(TestCase):
 
     def test00_simple(self):
         root = MyRoot()
-        server = Server(root)
+        server = WebServer(root)
 ##        server.start()
 
 
 
 
 if __name__ == '__main__':
-    unittest.main()
+    main()
