@@ -16,10 +16,10 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from the Standard Library
-import unittest
-from unittest import TestCase
+from unittest import TestCase, main
 
 # Import from itools
+from itools.handlers import ro_database
 from itools.xliff import XLFFile
 
 
@@ -29,9 +29,9 @@ class XLIFFTestCase(TestCase):
     def test_input(self):
         """Test input.
         """
-        xliff = XLFFile('tests/gettext_en_es.xlf')
+        xliff = ro_database.get_handler('tests/gettext_en_es.xlf', XLFFile)
         xliff.to_str()
 
 
 if __name__ == '__main__':
-    unittest.main()
+    main()
