@@ -15,8 +15,8 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from itools
+from itools.core import add_type, get_abspath
 from tmx import TMXFile, Sentence, TMXUnit, TMXNote
-from itools.utils import get_abspath, add_type
 from itools.xml import register_dtd
 
 
