@@ -21,7 +21,10 @@ Identifiers) as specified by RFC2396.
 # Import from itools
 import mailto
 from generic import Path, Reference, decode_query, encode_query
-from uri import get_reference, get_absolute_reference, get_absolute_reference2
+from generic import normalize_path
+from uri import get_reference, get_uri_name, get_uri_path
+from uri import resolve_uri, resolve_uri2, resolve_name
+from uri import get_host_from_authority
 from registry import register_scheme, get_scheme
 
 
@@ -31,8 +34,15 @@ __all__ = [
     'Reference',
     'decode_query',
     'encode_query',
-    'get_reference',
-    'get_absolute_reference',
-    'get_absolute_reference2',
     'register_scheme',
-    'get_scheme']
+    'get_scheme',
+    'normalize_path',
+    # New functional API
+    'get_reference',
+    'resolve_uri',
+    'resolve_uri2',
+    'resolve_name',
+    'get_uri_name',
+    'get_uri_path',
+    'get_host_from_authority',
+    ]
