@@ -48,68 +48,49 @@ from copy import copy
 from urlparse import urlsplit, urlunsplit
 from urllib import quote_plus, unquote, unquote_plus
 
-
-##########################################################################
-# Authority
-##########################################################################
-
-class Authority(object):
-    """There are two types of authorities: registry based and server-based;
-    right now only server-based are supported (XXX).
-
-    The userinfo component could be further processed.
-    """
-
-    __slots__ = ['userinfo', 'host', 'port']
-
-
-    def __init__(self, auth):
-        # The userinfo
-        if '@' in auth:
-            self.userinfo, auth = auth.split('@', 1)
-        else:
-            self.userinfo = None
-        # host:port
-        if ':' in auth:
-            self.host, self.port = auth.split(':', 1)
-        else:
-            self.host = auth
-            self.port = None
-
-
-    def __str__(self):
-        # userinfo@host
-        if self.userinfo is not None:
-            auth = '%s@%s' % (self.userinfo, self.host)
-        else:
-            auth = self.host
-        # The port
-        if self.port is not None:
-            return auth + ':' + self.port
-        return auth
-
-
-    def __eq__(self, other):
-        return str(self) == str(other)
-
-
-    def __ne__(self, other):
-        return str(self) != str(other)
-
-
-    def __hash__(self):
-        return hash(str(self))
-
-
-    def __nonzero__(self):
-        return bool(str(self))
-
+# Import from itools
+from itools.core import freeze
 
 
 #########################################################################
 # Path
 ##########################################################################
 
+def _normalize_path(path):
+    if type(path) is not str:
+        raise TypeError, 'path must be an string, not a %s' % type(path)
+
+    # Does the path start and/or end with an slash?
+    startswith_slash = endswith_slash = False
+    if path:
+        startswith_slash = (path[0] == '/')
+        if path[-1] == '/':
+            endswith_slash = True
+        elif len(path) > 1 and path[-2] == '/' and path[-1] == '.':
+            endswith_slash = True
+
+    # Reduce '//', '/./' and 'a/..'
+    stack = []
+    for name in path.split('/'):
+        # Reduce '//' and '/./' to '/'
+        if name == '' or name == '.':
+            continue
+        # Reduce 'a/..' to ''
+        if name == '..' and stack and stack[-1] != '..':
+            stack.pop()
+        else:
+            stack.append(name)
+    path = stack
+
+    # Absolute path: remove '..' at the beginning
+    if startswith_slash:
+        while path and path[0] == '..':
+            del path[0]
+
+    # Ok
+    return startswith_slash, path, endswith_slash
+
+
 def normalize_path(path):
     """Normalize the path (we don't use os.path because on Windows it
     converts forward slashes to back slashes).
@@ -122,66 +103,21 @@ def normalize_path(path):
       '/../a/b/c ' -> '/a/b/c'
       '.'          -> ''
     """
-    if not isinstance(path, (str, unicode)):
-        raise TypeError, 'path must be an string, not a %s' % type(path)
+    startswith_slash, path, endswith_slash = _normalize_path(path)
 
-    # Does the path start by an slash? i.e.: is it absolute?
-    startswith_slash = path.startswith('/')
-
-    # Does the path end by an slash? (relevant to resolve URLs)
-    endswith_slash = path.endswith('/') or path.endswith('/.')
-
-    # Split the path http://a//
-    path = path.split('/')
-
-    # Transform '//' and '/./' to '/'
-    path = [ x for x in path if x not in ('', '.') ]
-
-    # Transform 'a/..' to ''
-    stack = []
-    for name in path:
-        if name == '..' and stack and stack[-1] != '..':
-            stack.pop()
-        else:
-            stack.append(name)
-    path = stack
-
-    # Absolute or Relative
-    if startswith_slash:
-        # Absolute path, remove '..' at the beginning
-        while path and path[0] == '..':
-            path = path[1:]
-
-    path = '/'.join(path)
+    # len > 0
     if path:
+        path = '/'.join(path)
         if startswith_slash:
             path = '/' + path
         if endswith_slash:
             path = path + '/'
         return path
-    else:
-        if startswith_slash:
-            return '/'
-        return ''
 
-
-
-class Segment(str):
-
-    __slots__ = []
-
-    def get_name(self):
-        return self.split(';', 1)[0]
-
-    name = property(get_name, None, None, '')
-
-
-    def get_params(self):
-        if ';' in self:
-            return self.split(';')[1:]
-        return []
-
-    params = property(get_params, None, None, '')
+    # len = 0
+    if startswith_slash:
+        return '/'
+    return ''
 
 
 
@@ -198,28 +134,29 @@ class Path(list):
 
 
     def __init__(self, path):
-        if isinstance(path, tuple) or isinstance(path, list):
-            path = '/'.join([ str(x) for x in path ])
+        if type(path) is str:
+            startswith_slash, path, endswith_slash = _normalize_path(path)
+            self.startswith_slash = startswith_slash
+            self.endswith_slash = bool(path) and endswith_slash
+        elif type(path) is Path:
+            self.startswith_slash = path.startswith_slash
+            self.endswith_slash = path.endswith_slash
+        else:
+            # XXX Here the path is not normalized:
+            #
+            #   >>> print Path(['a', '..', 'b'])
+            #   a/../b
+            self.startswith_slash = False
+            self.endswith_slash = False
+            path = [ str(x) for x in path ]
 
-        path = normalize_path(path)
-
-        # Absolute or relative
-        self.startswith_slash = path.startswith('/')
-        if self.startswith_slash:
-            path = path[1:]
-        self.endswith_slash = path.endswith('/')
-        if self.endswith_slash:
-            path = path[:-1]
-
-        if path != '':
-            path = [ Segment(x) for x in path.split('/') ]
-            list.__init__(self, path)
+        list.__init__(self, path)
 
 
     def __getslice__(self, a, b):
-         slice = Path(list.__getslice__(self, a, b))
-         slice.startswith_slash = self.startswith_slash
-         return slice
+        slice = Path(list.__getslice__(self, a, b))
+        slice.startswith_slash = self.startswith_slash
+        return slice
 
 
     def __add__(self, path):
@@ -234,15 +171,13 @@ class Path(list):
 
 
     def __str__(self):
-        path = ''
-        if self.startswith_slash:
-            path = '/'
-        path += '/'.join([ str(x) for x in self ])
+        path = '/' if self.startswith_slash else ''
+
+        path += '/'.join(self)
         if self.endswith_slash:
             path += '/'
-        if len(path) == 0:
-            return '.'
-        return path
+
+        return path if path else '.'
 
 
     def __ne__(self, other):
@@ -273,7 +208,7 @@ class Path(list):
 
     def get_name(self):
         if len(self) > 0:
-            return self[-1].name
+            return self[-1]
         return ''
 
 
@@ -284,16 +219,20 @@ class Path(list):
           Path('/a/b').resolve('c') => Path('/a/c')
           Path('/a/b/').resolve('c') => Path('/a/b/c')
         """
-        if not isinstance(path, Path):
+        if type(path) is not Path:
             path = Path(path)
 
-        if path.is_absolute():
+        if path.startswith_slash:
             return path
 
+        self_str = str(self)
         if self.endswith_slash:
-            return Path('%s/%s' % (self, path))
+            return Path('%s%s' % (self_str, path))
 
-        return Path('%s/../%s' % (self, path))
+        if self_str == '.':
+            return path
+
+        return Path('%s/../%s' % (self_str, path))
 
 
     def resolve2(self, path):
@@ -307,13 +246,39 @@ class Path(list):
 
         Very, very practical.
         """
-        if not isinstance(path, Path):
+        if type(path) is not Path:
             path = Path(path)
 
-        if path.is_absolute():
+        if path.startswith_slash:
             return path
 
-        return Path('%s/%s' % (self, path))
+        # Resolve
+        new_path = Path(self)
+        new_path.startswith_slash = self.startswith_slash
+        new_path.endswith_slash = path.startswith_slash
+        for name in path:
+            if name == '..':
+                if new_path:
+                    new_path.pop()
+                elif not new_path.startswith_slash:
+                    new_path.append(name)
+            else:
+                new_path.append(name)
+        return new_path
+
+
+    def resolve_name(self, name):
+        """This is a particular case of the 'resolve2' method, where the
+        reference is known to be a relative path of length = 1.
+        """
+        if not isinstance(name, str):
+            raise TypeError, 'unexpected value "%s"' % repr(name)
+
+        # Relative path
+        path = copy(self)
+        path.append(name)
+        path.endswith_slash = False
+        return path
 
 
     def get_prefix(self, path):
@@ -324,7 +289,7 @@ class Path(list):
 
         XXX When there are parameters (e.g. a/b;lang=es/c) it is undefined.
         """
-        if not isinstance(path, Path):
+        if type(path) is not Path:
             path = Path(path)
 
         i = 0
@@ -338,12 +303,12 @@ class Path(list):
         the complement of 'resolve2'. So, if 'x = a.get_pathto(b)', then
         'b = a.resolve2(x)'.
         """
-        if not isinstance(path, Path):
+        if type(path) is not Path:
             path = Path(path)
 
         prefix = self.get_prefix(path)
         i = len(prefix)
-        return Path(((['..'] * len(self[i:])) + path[i:]) or ['.'])
+        return Path(((['..'] * len(self[i:])) + path[i:]) or [])
 
 
     def get_pathtoroot(self):
@@ -499,7 +464,7 @@ class Reference(object):
         if path == '.':
             path = ''
         query = encode_query(self.query)
-        reference = urlunsplit((self.scheme, str(self.authority), path, query,
+        reference = urlunsplit((self.scheme, self.authority, path, query,
                                 self.fragment))
         if reference == '':
             if self.fragment is not None:
@@ -536,45 +501,40 @@ class Reference(object):
 
         # Network path
         if reference.authority:
-            return Reference(self.scheme,
-                             copy(reference.authority),
+            return Reference(self.scheme, reference.authority,
                              copy(reference.path),
                              copy(reference.query),
                              reference.fragment)
 
         # Absolute path
         if reference.path.is_absolute():
-            return Reference(self.scheme,
-                             copy(self.authority),
+            return Reference(self.scheme, self.authority,
                              copy(reference.path),
                              copy(reference.query),
                              reference.fragment)
 
         # Internal references
-        if isinstance(reference, EmptyReference):
-            return Reference(self.scheme,
-                             copy(self.authority),
+        if reference is empty_reference:
+            return Reference(self.scheme, self.authority,
                              copy(self.path),
                              self.query.copy(),
                              None)
 
         if reference.fragment and not reference.path and not reference.query:
-            return Reference(self.scheme,
-                             copy(self.authority),
+            return Reference(self.scheme, self.authority,
                              copy(self.path),
                              copy(self.query),
                              reference.fragment)
 
         # Relative path
-        return Reference(self.scheme,
-                         copy(self.authority),
+        return Reference(self.scheme, self.authority,
                          self.path.resolve(reference.path),
                          copy(reference.query),
                          reference.fragment)
 
 
     def resolve2(self, reference):
-        """This is much like 'resolv', but uses 'Path.resolve2' method
+        """This is much like 'resolve', but uses 'Path.resolve2' method
         instead.
 
         XXX Too much code is duplicated, the only difference beween 'resolve'
@@ -589,43 +549,46 @@ class Reference(object):
 
         # Network path
         if reference.authority:
-            return Reference(self.scheme,
-                             copy(reference.authority),
+            return Reference(self.scheme, reference.authority,
                              copy(reference.path),
                              copy(reference.query),
                              reference.fragment)
 
         # Absolute path
         if reference.path.is_absolute():
-            return Reference(self.scheme,
-                             copy(self.authority),
+            return Reference(self.scheme, self.authority,
                              copy(reference.path),
                              copy(reference.query),
                              reference.fragment)
 
         # Internal references
-        if isinstance(reference, EmptyReference):
-            return Reference(self.scheme,
-                             copy(self.authority),
+        if reference is empty_reference:
+            return Reference(self.scheme, self.authority,
                              copy(self.path),
                              copy(self.query),
                              None)
 
         if reference.fragment and not reference.path and not reference.query:
-            return Reference(self.scheme,
-                             copy(self.authority),
+            return Reference(self.scheme, self.authority,
                              copy(self.path),
                              copy(self.query),
                              reference.fragment)
 
         # Relative path
-        return Reference(self.scheme,
-                         copy(self.authority),
+        return Reference(self.scheme, self.authority,
                          self.path.resolve2(reference.path),
                          copy(reference.query),
                          reference.fragment)
 
 
+    def resolve_name(self, name):
+        """This is a particular case of the 'resolve2' method, where the
+        reference is known to be a relative path of length = 1.
+        """
+        path = self.path.resolve_name(name)
+        return Reference(self.scheme, self.authority, path, {})
+
+
     def replace(self, **kw):
         """This method returns a new uri reference, equal to this one, but
         with the given keyword parameters set in the query.
@@ -659,7 +622,7 @@ class EmptyReference(Reference):
     scheme = None
     authority = None
     path = Path('')
-    query = None
+    query = freeze({})
     fragment = None
 
 
@@ -671,6 +634,9 @@ class EmptyReference(Reference):
         return ''
 
 
+empty_reference = EmptyReference()
+
+
 ##########################################################################
 # Factory
 ##########################################################################
@@ -679,19 +645,20 @@ class GenericDataType(object):
 
     @staticmethod
     def decode(data):
-        if isinstance(data, Path):
-            return Reference('', Authority(''), data, {}, None)
+        data_type = type(data)
+        if data_type is Path:
+            return Reference('', '', data, {}, None)
 
-        if not isinstance(data, (str, unicode)):
+        if data_type is not str:
             raise TypeError, 'unexpected %s' % type(data)
 
         # Special case, the empty reference
         if data == '':
-            return EmptyReference()
+            return empty_reference
 
         # Special case, the empty fragment
         if data == '#':
-            return Reference('', Authority(''), Path(''), {}, '')
+            return Reference('', '', Path(''), {}, '')
 
         # All other cases, split the reference in its components
         scheme, authority, path, query, fragment = urlsplit(data)
@@ -716,7 +683,6 @@ class GenericDataType(object):
             path = '/'
         # The authority
         authority = unquote(authority)
-        authority = Authority(authority)
         # The query
         try:
             query = decode_query(query)
