@@ -26,6 +26,8 @@ class Resource(object):
     """This is the base class for all web resources.
     """
 
+    __hash__ = None
+
     #######################################################################
     # API / Private
     #######################################################################
@@ -34,7 +36,7 @@ class Resource(object):
 
 
     def _get_resource(self, name):
-        raise LookupError
+        return None
 
 
     #######################################################################
@@ -45,9 +47,7 @@ class Resource(object):
             return Path('/')
         parent_path = self.parent.get_abspath()
 
-        return parent_path.resolve2(self.name)
-
-    abspath = property(get_abspath, None, None, '')
+        return parent_path.resolve_name(self.name)
 
 
     def get_canonical_path(self):
@@ -55,7 +55,7 @@ class Resource(object):
             return Path('/')
         parent_path = self.parent.get_canonical_path()
 
-        return parent_path.resolve2(self.name)
+        return parent_path.resolve_name(self.name)
 
 
     def get_real_resource(self):
@@ -75,21 +75,13 @@ class Resource(object):
         return self.get_abspath().get_pathto(handler.get_abspath())
 
 
-    def has_resource(self, path):
-        try:
-            self.get_resource(path)
-        except LookupError:
-            return False
-        return True
-
-
     def get_names(self, path='.'):
         resource = self.get_resource(path)
         return resource._get_names()
 
 
-    def get_resource(self, path):
-        if not isinstance(path, Path):
+    def get_resource(self, path, soft=False):
+        if type(path) is not Path:
             path = Path(path)
 
         if path.is_absolute():
@@ -103,6 +95,10 @@ class Resource(object):
 
         for name in path:
             resource = here._get_resource(name)
+            if resource is None:
+                if soft is True:
+                    return None
+                raise LookupError, 'resource "%s" not found' % path
             resource.parent = here
             resource.name = name
             here = resource
@@ -123,7 +119,7 @@ class Resource(object):
         raise NotImplementedError
 
 
-    def del_resource(self, path):
+    def del_resource(self, path, soft=False):
         raise NotImplementedError
 
 
@@ -222,21 +218,11 @@ class RootResource(AccessControl, Resource):
     #######################################################################
     # API / Error Pages
     #######################################################################
-    def unauthorized(self, context):
-        """Called on status 401 for replacing the body by an error page."""
-        return '401 Unauthorized'
-
-
-    def forbidden(self, context):
-        """Called on status 403 for replacing the body by an error page."""
-        return '403 Forbidden'
-
-
-    def not_found(self, context):
-        """Called on status 404 for replacing the body by an error page."""
-        return '404 Not Found'
-
-
-    def internal_server_error(self, context):
-        """Called on status 500 for replacing the body by an error page."""
-        return '500 Internal Server Error'
+    # FIXME These views should be defined in the site-root
+    # FIXME Implement a default behaviour for these views.
+    unauthorized = None # 401 Unauthorized
+    forbidden = None # 403 Forbidden
+    not_found = None # 404 Not Found
+    method_not_allowed = None # 405 Method Not Allowed
+    conflict = None # 409 Conflict
+    internal_server_error = None # 500 Internal Server Error
