@@ -15,10 +15,14 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+# Import from the Standard Library
+from copy import deepcopy
+
 # Import from itools
+from itools.core import freeze
 from itools.datatypes import Enumerate
 from itools.stl import stl
-from itools.uri import decode_query
+from itools.uri import decode_query, Reference
 from context import FormError
 
 
@@ -95,6 +99,36 @@ class BaseView(object):
         return self.title
 
 
+    #######################################################################
+    # Canonical URI for search engines
+    # "language" is by default because too widespreaded
+    canonical_query_parameters = freeze(['language'])
+
+
+    def get_canonical_uri(self, context):
+        """Return the same URI stripped from redundant view name, if already
+        the default, and query parameters not affecting the resource
+        representation.
+        Search engines will keep this sole URI when crawling different
+        combinations of this view.
+        """
+        uri = deepcopy(context.uri)
+        query = uri.query
+        # Remove the view name if default
+        view_name = context.view_name
+        if view_name:
+            resource = context.resource
+            if view_name == resource.get_default_view_name():
+                uri = uri.resolve2('..')
+        # Remove noise from query parameters
+        canonical_query_parameters = self.canonical_query_parameters
+        for parameter in query.keys():
+            if parameter not in canonical_query_parameters:
+                del query[parameter]
+        uri.query = query
+        return uri
+
+
 
 class BaseForm(BaseView):
 
@@ -132,79 +166,24 @@ class BaseForm(BaseView):
         return datatype.get_default()
 
 
-    def build_namespace(self, resource, context):
-        """This utility method builds a namespace suitable to use to produce
-        an HTML form. Its input data is a dictionnary that defines the form
-        variables to consider:
-
-          {'toto': Unicode(mandatory=True, multiple=False, default=u'toto'),
-           'tata': Unicode(mandatory=True, multiple=False, default=u'tata')}
-
-        Every element specifies the datatype of the field.
-        The output is like:
-
-            {<field name>: {'value': <field value>, 'class': <CSS class>}
-             ...}
-        """
-        # Figure out whether the form has been submit or not (FIXME This
-        # heuristic is not reliable)
-        schema = self.get_schema(resource, context)
-        submit = (context.request.method == 'POST')
-
-        # Build the namespace
-        namespace = {}
-        for name in schema:
-            datatype = schema[name]
-            is_mandatory = getattr(datatype, 'mandatory', False)
-            is_readonly = getattr(datatype, 'readonly', False)
-
-            cls = []
-            if is_mandatory:
-                cls.append('field_is_required')
-            if submit and not is_readonly:
-                try:
-                    value = context.get_form_value(name, type=datatype)
-                except FormError:
-                    cls.append('field_is_missing')
-                    if issubclass(datatype, Enumerate):
-                        value = datatype.get_namespace(None)
-                    else:
-                        value = context.get_form_value(name)
-                else:
-                    if issubclass(datatype, Enumerate):
-                        value = datatype.get_namespace(value)
-                    elif datatype.multiple:
-                        # XXX Done for table multilingual fields (fragile)
-                        value = value[0]
-                    else:
-                        value = datatype.encode(value)
-            else:
-                value = self.get_value(resource, context, name, datatype)
-                if issubclass(datatype, Enumerate):
-                    value = datatype.get_namespace(value)
-                else:
-                    value = datatype.encode(value)
-            cls = ' '.join(cls) or None
-            namespace[name] = {'name': name, 'value': value, 'class': cls}
-        return namespace
-
-
     def _get_action(self, resource, context):
-        """ default function to retrieve the name of the action from a form
+        """Default function to retrieve the name of the action from a form
         """
-        for name in context.get_form_keys():
-            if name.startswith(';'):
-                action = 'action_%s' % name[1:]
-                # Save the query of the action into context.form_query
-                if '?' in action:
-                    action, query = action.split('?')
-                    # Deserialize query using action specific schema
-                    schema = getattr(self, '%s_query_schema' % action, None)
-                    context.form_query = decode_query(query, schema)
-                context.form_action = action
-                break
-        else:
+        form = context.get_form()
+        action = form.get('action')
+        if action is None:
             context.form_action = 'action'
+            return
+
+        action = 'action_%s' % action
+        # Save the query of the action into context.form_query
+        if '?' in action:
+            action, query = action.split('?')
+            # Deserialize query using action specific schema
+            schema = getattr(self, '%s_query_schema' % action, None)
+            context.form_query = decode_query(query, schema)
+
+        context.form_action = action
 
 
     def get_action_method(self, resource, context):
@@ -255,21 +234,77 @@ class STLView(BaseView):
         if self.template is None:
             msg = "%s is missing the 'template' variable"
             raise NotImplementedError, msg % repr(self.__class__)
-        # XXX a handler actually
+        # XXX A handler actually
         return resource.get_resource(self.template)
 
 
     def GET(self, resource, context):
-        # Get the template
-        template = self.get_template(resource, context)
-
         # Get the namespace
         namespace = self.get_namespace(resource, context)
+        if isinstance(namespace, Reference):
+            return namespace
 
-        # Ok
+        # STL
+        template = self.get_template(resource, context)
         return stl(template, namespace)
 
 
 
 class STLForm(STLView, BaseForm):
-    pass
+
+    def get_namespace(self, resource, context, query=None):
+        """This utility method builds a namespace suitable to use to produce
+        an HTML form. Its input data is a dictionnary that defines the form
+        variables to consider:
+
+          {'toto': Unicode(mandatory=True, multiple=False, default=u'toto'),
+           'tata': Unicode(mandatory=True, multiple=False, default=u'tata')}
+
+        Every element specifies the datatype of the field.
+        The output is like:
+
+            {<field name>: {'value': <field value>, 'class': <CSS class>}
+             ...}
+        """
+        # Figure out whether the form has been submit or not (FIXME This
+        # heuristic is not reliable)
+        schema = self.get_schema(resource, context)
+        submit = (context.method == 'POST')
+
+        # Build the namespace
+        namespace = {}
+        for name in schema:
+            datatype = schema[name]
+            is_mandatory = getattr(datatype, 'mandatory', False)
+            is_readonly = getattr(datatype, 'readonly', False)
+
+            cls = []
+            if is_mandatory:
+                cls.append('field-is-required')
+            if submit and not is_readonly:
+                try:
+                    value = context.get_form_value(name, type=datatype)
+                except FormError:
+                    cls.append('field-is-missing')
+                    if issubclass(datatype, Enumerate):
+                        value = datatype.get_namespace(None)
+                    else:
+                        value = context.get_form_value(name)
+                else:
+                    if issubclass(datatype, Enumerate):
+                        value = datatype.get_namespace(value)
+                    elif datatype.multiple:
+                        # XXX Done for table multilingual fields (fragile)
+                        value = value[0]
+                    else:
+                        value = datatype.encode(value)
+            else:
+                value = self.get_value(resource, context, name, datatype)
+                if issubclass(datatype, Enumerate):
+                    value = datatype.get_namespace(value)
+                else:
+                    value = datatype.encode(value)
+            cls = ' '.join(cls) or None
+            namespace[name] = {'name': name, 'value': value, 'class': cls}
+        return namespace
+
