@@ -17,27 +17,16 @@
 # Import from itools
 from base import CatalogAware
 from catalog import Catalog, make_catalog
-from exceptions import XapianIndexError
-from fields import BaseField, register_field, get_field
-from fields import BoolField, KeywordField, IntegerField, TextField
+from queries import AllQuery, NotQuery, StartQuery, TextQuery
 from queries import RangeQuery, PhraseQuery, AndQuery, OrQuery
-from queries import AllQuery, NotQuery, StartQuery
+from utils import split_unicode
 
 
 __all__ = [
     'make_catalog',
     'Catalog',
     'CatalogAware',
-    # Exceptions
-    'XapianIndexError',
-    # Fields
-    'BaseField',
-    'TextField',
-    'KeywordField',
-    'IntegerField',
-    'BoolField',
-    'register_field',
-    'get_field',
+    'split_unicode',
     # Queries
     'RangeQuery',
     'PhraseQuery',
@@ -45,5 +34,7 @@ __all__ = [
     'OrQuery',
     'AllQuery',
     'NotQuery',
-    'StartQuery']
+    'StartQuery',
+    'TextQuery']
+
 
