@@ -15,7 +15,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from itools
-from itools.utils import get_abspath, add_type
+from itools.core import add_type, get_abspath
 from itools.xml import register_dtd
 from xliff import XLFFile, XLFUnit, XLFNote, File
 
