@@ -97,7 +97,7 @@ PyDocType_init (PyDocType * self, PyObject * args, PyObject * kwds)
   doctype = doctype_new (PubidLiteral, SystemLiteral, intSubset, &error_msg);
   if (!doctype)
     {
-      PyErr_Format (XMLError, error_msg);
+      PyErr_SetString (XMLError, error_msg);
       return -1;
     }
   self->doctype = doctype;
@@ -337,19 +337,14 @@ XMLParser_translate_STag (XMLParser * self)
           return NULL;
         }
       /* The tuple (uri, value) */
-      qname = Py_BuildValue ("(OO)", uri, name);
+      qname = Py_BuildValue ("(NN)", uri, name);
       if (qname == NULL)
         {
           Py_DECREF (attributes);
           Py_DECREF (uri);
-          Py_DECREF (uri);
-          Py_DECREF (name);
           Py_DECREF (name);
           return NULL;
         }
-      /* Py_Buildvalue increments the counters */
-      Py_DECREF (uri);
-      Py_DECREF (name);
       /* The value */
       value = PyString_FromString ((char *) (attribute->value->str));
       if (value == NULL)
@@ -362,12 +357,11 @@ XMLParser_translate_STag (XMLParser * self)
       if (PyDict_SetItem (attributes, qname, value) != 0)
         {
           Py_DECREF (attributes);
-          /* XXX two times ??? */
           Py_DECREF (qname);
           Py_DECREF (value);
           return NULL;
         }
-      /* PyDict_SetItem increments the counters ?? */
+      /* PyDict_SetItem increments the counters */
       Py_DECREF (qname);
       Py_DECREF (value);
     }
@@ -771,7 +765,8 @@ initparser (void)
   PyModule_AddObject (module, "DocType", (PyObject *) & PyDocTypeType);
 
   /* Register exceptions */
-  XMLError = PyErr_NewException ("itools.xml.parser.XMLError", NULL, NULL);
+  XMLError = PyErr_NewException ("itools.xml.parser.XMLError",
+                                 PyExc_StandardError, NULL);
   Py_INCREF (XMLError);
   PyModule_AddObject (module, "XMLError", XMLError);
 
